#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2009-01-19
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: analysis.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/api/analysis.pm $
#
package npg_qc::api::analysis;
use strict;
use warnings;
use base qw(npg_qc::api::base);
use Class::Std;
use Carp qw{cluck confess croak carp};
use English qw{-no_match_vars};
use Readonly;
use npg_qc::api::analysis_lane;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

{
  ## no critic (ProhibitUnusedVariables)
  my %id_run_of         :ATTR( :get<id_run>,         :set<id_run> );
  my %id_analysis_of    :ATTR( :get<id_analysis>,    :set<id_analysis> );
  my %date_of           :ATTR( :get<date>,           :set<date> );
  my %folder_of         :ATTR( :get<folder>,         :set<folder> );
  my %iscurrent_of      :ATTR( :get<iscurrent>,      :set<iscurrent> );
  my %fc_yield_of       :ATTR( :get<fcyield>,        :set<fcyield> );
  my %response_of       :ATTR( :get<response>,       :set<response> );
  my %analyses_of       :ATTR( :get<analyses>,       :set<analyses> );
  my %analysis_lanes_of :ATTR( :get<analysis_lanes>, :set<analysis_lanes> );
  ## use critic

  sub fields {
    return (qw(id_analysis id_run date folder iscurrent fcyield));
  };

  sub id_analysis {
    my ($self, $id_analysis) = @_;
    return $self->store_and_return('id_analysis', $id_analysis);
  }
  sub id_run {
    my ($self, $id_run) = @_;
    return $self->store_and_return('id_run', $id_run);
  }
  sub date {
    my ($self, $date) = @_;
    return $self->store_and_return('date', $date);
  }
  sub folder {
    my ($self, $folder) = @_;
    return $self->store_and_return('folder', $folder);
  }
  sub iscurrent {
    my ($self, $iscurrent) = @_;
    return $self->store_and_return('iscurrent', $iscurrent);
  }
  sub fcyield {
    my ($self, $fcyield) = @_;
    return $self->store_and_return('fcyield', $fcyield);
  }

  sub get_analysis_data {
    my ($self, $runs) = @_;

    my $util   = $self->util();
    my $parser = $util->parser();

    my $xml = qq{<?xml version="1.0" encoding="utf-8"?><root><runs>\n};
    foreach my $r (@{$runs}) {
      $xml .= qq{<run id_run="$r" />};
    }
    $xml .= qq{\n</runs></root>};
    eval {
      my $parsed_xml = $parser->parse_string($xml);
      my $response = $util->post(
        $util->base_uri().q{analysis.xml},
        [
          'Content_Type'   => 'application/xml',
  			  'Content_Length' => length $xml,
  			  'Content'        => $xml,
  			  'Accept'         => 'text/xml',
			  ]
      );
      if ($response->is_success()) {
        my $parsed_response = $parser->parse_string($response->content());
        $self->set_response($parsed_response);
      } else {
        croak $response->status_line();
      }
      1;
    } or do {
      croak $EVAL_ERROR;
    };
    return $self->build_objects();
  }

  sub build_objects {
    my ($self) = @_;
    my $pkg = ref$self;
    my $response = $self->get_response();
    my $analyses = $response->getElementsByTagName('analysis');
    $self->set_analyses([]);
    foreach my $a (@{$analyses}) {

      my $object = $pkg->new({ util => $self->util() });
      foreach my $f ($self->fields()) {
        if ($f ne 'fcyield') {
          $object->$f($a->getAttribute($f));
        } else {
	  $object->$f($a->getAttribute('FC_yield'));
	}
      }
      push @{$self->get_analyses()}, $object;
      $object->set_analysis_lanes([]);

      my $lanes = $a->getElementsByTagName('analysis_lane');
      foreach my $l (@{$lanes}) {
        my $l_object = npg_qc::api::analysis_lane->new({ util => $self->util() });
	foreach my $f ($l_object->fields()) {
	  $l_object->$f($l->getAttribute($f));
	}
	push @{$object->get_analysis_lanes()}, $l_object;
      }

    }
    return 1;
  }
  sub store_and_return {
    my ($self, $method, $value) = @_;
    if ($value) {
      my $set_method = 'set_'.$method;
      $self->$set_method($value);
    }
    my $get_method = 'get_'.$method;
    return $self->$get_method();
  }

}
1;
__END__

=head1 NAME

npg_qc::api::analysis

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

  my $oAnalysis = npg_qc::api::analysis->new({
    util => $oNPG_QC_Util,
  });

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - list of accessors for this object

=head2 id_analysis - accessor for id_analysis

=head2 id_run - accessor for id_run

=head2 date - accessor for date

=head2 folder - accessor for folder

=head2 iscurrent - accessor for iscurrent

=head2 fcyield - accessor for fcyield

=head2 get_analysis_data - method to call to obtain run analysis data. Calls build_objects to construct the objects in the correct data structure

  eval { $oAnalysis->get_analysis_data($aRunIDs); 1; } or do { croak $EVAL_ERROR; };

=head2 build_objects - method used to construct the objects from the fetched XML

=head2 store_and_retrieve - refactored code for all accessors to use

=head2 store_and_return

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
English
Class::Std
Carp
npg::api:base
npg_qc::api::analysis_lane

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
