#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-02-19
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: Matrix.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/api/loader/Matrix.pm $
#
package npg_qc::api::loader::Matrix;
use strict;
use warnings;
use Moose;
use Carp qw{carp cluck croak confess};
use English qw{-no_match_vars};
use Readonly;
use Perl6::Slurp;

extends qw{npg_qc::api::loader::base};

Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $BASE_ORDER => [ qw(A C T G) ];
Readonly::Scalar our $SECOND_READ_NUMER_INDEX_RUN => 3;

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::api::loader::Matrix

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 file_list

matrix file list
 
=cut
has 'file_list'    => (isa           => q{ArrayRef},
                       is            => q{rw},
                       lazy_build    => 1,
                       documentation => q{matrix file name list},
                      );

sub _build_file_list {
  my $self = shift;

  my @file_list = ();

  my $folder_path = $self->bustard_path() . q{/Matrix};
  opendir my $dh, $folder_path or ( carp "Cannot open $folder_path" && return \@file_list );
  @file_list = map { $folder_path.qq{/$_} } grep { /s_\d+_\d+_matrix[.]txt/mxs } sort readdir $dh;
  closedir $dh or croak qq(Cannot close $folder_path);

  return \@file_list;
}

=head2 run

=cut

sub run {
  my ($self) = @_;

  $self->log(q{Loading Illumina matrix files for Run } . $self->id_run() . q{ into QC database});

  my $file_list = $self->file_list();
  $self->log('There are '.scalar @{$file_list}. ' files to process');

  foreach my $matrix_file (@{$file_list}){

    $self->_process_matrix( $matrix_file );
  }

  $self->schema->txn_commit();

  return 1;
}

sub _process_matrix {
    my ($self, $file_url) = @_;

    my $id_run = $self->id_run();

    my ($position, $cycle_or_read) = $file_url =~ /s_(\d{1})_(\d+)_matrix[.]txt/xms;
    $cycle_or_read += 0;

    my $field_db = {id_run => $id_run,
                    lane   => $position,
                    cycle  => $cycle_or_read,
                   };

    my @lines    = split /\n/xms, slurp $file_url;

    my @base_list;
    my @values;
    my $cycle_number_in_filename;

    foreach my $line (@lines) {
      next if ($line =~ /\A\#/xms);
      if($line =~ /\A>/xms){
        push @base_list, substr $line, 2, 1;
        $cycle_number_in_filename = 1;
      }else{
         my @values_per_line = split /\s+/xms, $line;
         push @values, @values_per_line;
      }
    }

    if( ! (scalar @base_list) ){
        @base_list = @{$BASE_ORDER};
    }

    if(! $cycle_number_in_filename){
       $cycle_or_read = $self->_second_cycle_number_by_read($cycle_or_read);
       $field_db->{cycle} = $cycle_or_read;
    }

    foreach my $base (@base_list) {

      $field_db->{base} = $base;
      $field_db->{red1} = shift @values;
      $field_db->{red2} = shift @values;
      $field_db->{green1} = shift @values;
      $field_db->{green2} = shift @values;

      $self->schema->resultset('FrequencyResponseMatrix')->update_or_create( $field_db );
    }

    return 1;
}

sub _second_cycle_number_by_read {
   my ($self, $read) = @_;

   if( $read == 1 ){

      my @range= $self->read1_cycle_range();
      return $range[0] + 1;
   }elsif( $read == $SECOND_READ_NUMER_INDEX_RUN || ($read == 2 && !$self->is_indexed())){

     my @range= $self->read2_cycle_range();
      return $range[0] + 1;
   }elsif($read == 2 && $self->is_indexed()){

      my @range = $self->indexing_cycle_range();
      return $range[0] + 1;
   }
   return;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item MooseX::Getopt

=item npg_qc::api::loader::base

=item Perl6::Slurp

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
