#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-02-19
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: Signal_Mean.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/api/loader/Signal_Mean.pm $
#
package npg_qc::api::loader::Signal_Mean;
use strict;
use warnings;
use Moose;
use Carp qw{carp cluck croak confess};
use English qw{-no_match_vars};
use Readonly;
use Perl6::Slurp;

extends qw{npg_qc::api::loader::base};

Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $VALUE_FOR_DATA_MINING_EFFICIENCY => -1;

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::api::loader::Signal_Mean

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 file_name

signal mean test file name
 
=cut
has 'file_name'    => (isa           => q{Str},
                       is            => q{rw},
                       lazy_build    => 1,
                       documentation => q{file name for signal mean text file},
                      );

sub _build_file_name {
  my $self = shift;
  return $self->bustard_path().q{/Signal_Means.txt};
}

=head2 run

=cut

sub run {
  my ($self) = @_;

  $self->log(q{Loading Illumina signal mean file for Run } . $self->id_run() . q{ into QC database});

  my $signal_mean_file = $self->file_name();
  if(! -e $signal_mean_file){
    $self->log("Signal mean file not exists!\n$signal_mean_file");
    return 1;
  }

  my $file_content = slurp $signal_mean_file;

  my @lines  = split /\n/xms, $file_content;
  my @fields = qw(position cycle all_a all_c all_g all_t call_a call_c call_g call_t base_a base_c base_g base_t);
  foreach my $line (@lines) {
    next if ($line =~ /\A\#/xms);

    my @values_per_lane_cycle = split q{ }, $line;
    my %values_hash = ();
    @values_hash{@fields} = @values_per_lane_cycle;
    $values_hash{id_run} = $self->id_run();
    $self->schema->resultset('SignalMean')->update_or_create( %values_hash );
  }

  $self->schema->txn_commit();

  return 1;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item MooseX::Getopt

=item npg_qc::api::loader::base

=item Perl6::Slurp

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
