#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-01-27
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: Cluster_Density.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/api/monitoring/Cluster_Density.pm $
#

package npg_qc::api::monitoring::Cluster_Density;

use strict;
use warnings;
use Moose;
use Carp;
use English qw{-no_match_vars};
use npg_qc::api::util;
use npg_qc::api::loader::Cluster_Density;
use Readonly;

with 'npg_qc::api::monitoring::Role';

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $GLOB_PATTERN => q{*/*/Data/reports/NumClusters\ By\ Lane.xml};

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::api::monitoring::Cluster_Density

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 _build_glob_pattern

lazy build method for glob_pattern defined in the role
 
=cut

sub _build_glob_pattern {

   return $GLOB_PATTERN;
}

=head2 _build_runlist_db

lazy build method

id_run list already in the database
 
=cut

sub _build_runlist_db {
   my $self = shift;

   my $dbh = $self->dbh();

   my $runlist_db = {};

   my $query = q{SELECT id_run, count(*) AS c FROM cluster_density GROUP BY id_run HAVING c = 16};

   my $rows_ref;

   eval {
      $rows_ref = $dbh->selectall_arrayref($query);
      1;
   } or do {
      croak "Problems to get a list of runs in database with cluster density data: $EVAL_ERROR";
   };

   if(!$rows_ref){
     return $runlist_db;
   }

   foreach my $row_ref (@{$rows_ref}){
     $runlist_db->{$row_ref->[0]} = 1;
   }

   return $runlist_db;
}


=head2 run

get the list of runfolder with cluster density xml file
save them into database if not there
 
=cut
sub run {
  my $self = shift;

  $self->log('Finding runfolder with cluster density files and load them into QC database');
  $self->log('There are '.scalar @{$self->runfolder_list_todo}.' runs to do' );

  foreach my $runfolder_path (@{$self->runfolder_list_todo}){

     $self->save_one_run_data($runfolder_path);
  }
  return 1;
}

=head2 save_one_run_data

given one runfolder path save the data into database
 
=cut
sub save_one_run_data{
  my ($self, $runfolder_path) = @_;
  eval{
    my $loader = npg_qc::api::loader::Cluster_Density->new({
                       runfolder_path => $runfolder_path,
                       util => $self->util(),
    });
    $loader->run();
    1;
  } or do {
    carp "Loading run $runfolder_path failed: $EVAL_ERROR";
  };
  return 1;
}

no Moose;
__PACKAGE__->meta->make_immutable;

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item MooseX::Getopt

=item Carp

=item English qw{-no_match_vars}

=item npg_qc::api::util

=item npg_qc::api::loader::cluster_density

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
