#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-19
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: tile.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/api/run/lane/tile.pm $
#
package npg_qc::api::run::lane::tile;
use strict;
use warnings;
use Class::Std;
use Carp;
use base qw(npg_qc::api::run::lane);
use Readonly;
use JSON;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

{
  ##no critic (ProhibitUnusedVariables)
  my %id_run_tile_of  :ATTR( :get<id_run_tile>,  :set<id_run_tile>  );
  my %tile_of         :ATTR( :get<tile>,         :set<tile>         );
  my %row_of          :ATTR( :get<tile_row>,     :set<tile_row>     );
  my %col_of          :ATTR( :get<tile_col>,     :set<tile_col>     );
  my %avg_newz_of     :ATTR( :get<avg_newz>,     :set<avg_newz>     );
  my %tile_qc_data_of :ATTR( :get<tile_qc_data>, :set<tile_qc_data> );
  ##use critic

  sub fields {
    return qw(
              id_run_tile
              tile
              tile_row
              tile_col
              avg_newz
              tile_qc_data
            );
  }

  sub get_row {
    my ($self) = @_;
    return $self->get_tile_row();
  }

  sub get_col {
    my ($self) = @_;
    return $self->get_tile_col();
  }

}
1;
__END__
=head1 NAME

npg_qc::api::run::lane::tile

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - Constructor

  my $oRunLaneTile = npg_qc::api::run::lane::tile->new({'util' => $oUtil,});

=head2 fields - returns an array of the fields in the run_tile table

  my $aFields = $oRunLaneTile->fields();

=head2 get_lane - accessor to obtain the lane number

  my $lane_number = $oRunLaneTile->get_lane();

=head2 id_run - accessor to obtain the id_run (note this is the real id_run for the end)

  my $id_run = $oRunLaneTile->id_run();

=head2 get_read_end - returns 1 or 2 dependent whether this is the first or second end

=head2 get_id_run_tile - returns the value for the accessor for this tile
=head2 get_tile
=head2 get_row
=head2 get_col
=head2 get_avg_newz

=head2 get_tile_qc_data - returns a href of the tile qc data that is provided by default from the web service

  my $hTileQCData = $oRunLaneTile->get_tile_qc_data();
  
  $hTileQCData = {
   clusters_raw => "191411",
   pf_cluster_count => "60581",
   pc_pf_cluster => "31.65",
   av_1st_cycle_int_pf => "241.00",
   av_pc_int_after_20_cycles_pf => "46.39",
   pc_align_pf => "96.2",
   av_align_score_pf => "48.9",
   pc_err_rate_pf => "0.92"
  }

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
npg_qc::api::run::lane
Readonly
JSON

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
