#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-19
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: run_tile.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/api/run_tile.pm $
#
package npg_qc::api::run_tile;
use strict;
use warnings;
use Class::Std;
use Carp;
use base qw(npg_qc::api::base);
use Readonly;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

{
  ## no critic (ProhibitUnusedVariables)
  my %run_tiles_of   :ATTR( :get<run_tiles>, :set<run_tiles> );
  my %id_run_tile_of :ATTR( :get<id_run_tile>, :set<id_run_tile> );
  my %id_run_of      :ATTR( :get<id_run>, :set<id_run> );
  my %position_of    :ATTR( :get<position>, :set<position> );
  my %tile_of        :ATTR( :get<tile>, :set<tile> );
  my %end_of         :ATTR( :get<end>,  :set<end> );
  my %row_of         :ATTR( :get<row>,  :set<row> );
  my %col_of         :ATTR( :get<col>,  :set<col> );
  my %avg_newz_of    :ATTR( :get<avg_newz>,  :set<avg_newz> );
  ## use critic

  sub fields {
    return qw(
              id_run_tile
              id_run
              position
              tile
              end
              row
              col
              avg_newz
            );
  }
  sub id_run_tile {
    my ($self, $id_run_tile) = @_;
    if ($id_run_tile) {
      $self->set_id_run_tile($id_run_tile);
    }
    return $self->get_id_run_tile();
  }
  sub id_run {
    my ($self, $id_run) = @_;
    if ($id_run) {
      $self->set_id_run($id_run);
    }
    return $self->get_id_run();
  }
  sub position {
    my ($self, $position) = @_;
    if ($position) {
      $self->set_position($position);
    }
    return $self->get_position();
  }
  sub tile {
    my ($self, $tile) = @_;
    if ($tile) {
      $self->set_tile($tile);
    }
    return $self->get_tile();
  }

  sub row {
    my ($self, $row) = @_;
    if ($row) {
      $self->set_row($row);
    }
    return $self->get_row();
  }

  sub col {
    my ($self, $col) = @_;
    if ($col) {
      $self->set_col($col);
    }
    return $self->get_col();
  }

  sub end {
    my ($self, $end) = @_;
    if ($end) {
      $self->set_end($end);
    }
    return $self->get_end();
  }
  sub avg_newz{
    my ($self, $avg_newz) = @_;
    if ($avg_newz) {
      $self->set_avg_newz($avg_newz);
    }
    return $self->get_avg_newz();
  }
  sub run_tiles {
    my ($self) = @_;
    my $util = $self->util();
    my $pkg        = ref $self;
    my ($obj_type) = $pkg =~ /([^:]+)$/mxs;
    my $obj_pk     = $self->primary_key();
    my $obj_uri    = sprintf '%s/%s', $util->base_uri(), $obj_type;
    my $response   = $util->get($obj_uri);

    if(!$response->is_success()) {
      croak $response->status_line();
    }
    $self->set_run_tiles($util->parser->parse_string($response->content()));
    my $run_tiles = $self->get_run_tiles()->getElementsByTagName('run_tiles')->[0];

    return [map { $self->new_from_xml($pkg, $_) } $run_tiles->getElementsByTagName('run_tile')];
  }

  sub movez_id_runs {
    my ($self) = @_;
    my $util = $self->util();
    my $pkg        = ref $self;
    my ($obj_type) = $pkg =~ /([^:]+)$/mxs;
    my $obj_uri    = sprintf '%s/%s/%s', $util->base_uri(), $obj_type, q{;list_movez_id_run_xml};
    my $response   = $util->get($obj_uri);

    if(!$response->is_success()) {
      croak $response->status_line();
    }
    my $response_content_doc = $util->parser->parse_string($response->content());
    my $id_runs = $response_content_doc->getElementsByTagName('id_runs')->[0];

    my %return_id_runs = ();

    foreach my $id_run (@{$id_runs->getElementsByTagName('id_run')}){
      $return_id_runs{$id_run->getAttribute('id')} = $id_run->getAttribute('tiles');
    }

    return \%return_id_runs;
  }

}
1;
__END__

=head1 NAME

npg_qc::api::run_tile

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2       avg_newz  - access methods
=head2       movez_id_runs - by web services, get a xml file with a list of id_runs already in the database with run_log finished, and store id_runs into an array

=head2 new - Constructor

  my $oRunTile = npg_qc::api::run_tile->new({'util' => $oUtil,});

=head2 fields - returns an array of the fields in the run_tile table

  my $aFields = $oRunTile->fields();

=head2 run_tiles - fetches all run tiles from the npg_qc application, and turns into array of npg_qc::api::run_tile objects

  my $aRunTiles = $oRunTile->run_tiles();

=head2 id_run_tile - accessor for id_run_tile. takes an id as an optional parameter (which it stores) and returns the id_run_tile

  my $id_run_tile = $oRunTile->id_run_tile($id_run_tile);

=head2 id_run - accessor for id_run. takes an id as an optional parameter (which it stores) and returns the id_run

  my $id_run = $oRunTile->id_run($id_run);

=head2 position - accessor for position. takes a position as an optional parameter (which it stores) and returns the position

  my $position = $oRunTile->position($position);

=head2 tile - accessor for tile. takes a tile number as an optional parameter (which it stores) and returns the tile

  my $tile = $oRunTile->tile($tile);

=head2 end - accessor for end. takes an end number as an optional parameter (which it stores) and returns the end

  my $end = $oRunTile->end($end);

=head2 row - accessor for row. takes a row number as an optional parameter (which it stores) and returns the row

  my $row = $oRunTile->row($row);

=head2 col - accessor for col. takes a col number as an optional parameter (which it stores) and returns the col

  my $col = $oRunTile->col($col);

=head2 util - util instance for this resource set

  Returns a cached object if one exists, or creates and caches a new one

  my $oUtil = $oDerived->util();

=head2 new_from_xml - create and populate an npg::api::* from a XML::LibXML::Element

  my $oObj = $oDerived->new_from_xml('npg::api::<package>', $oXMLElement);

=head2 hasa - makes one:one accessors on compilation

 package npg::api::<base-subclass>;
 use strict;
 use warnings;

 __PACKAGE__->hasa(['child1', 'child2', {'current_child3' => 'child3-package'}]);

=head2 hasmany - makes one:many accessors on compilation

 package npg::api::<base-subclass>;
 use strict;
 use warnings;

 __PACKAGE__->hasmany(['child1', 'child2', {'current_child3' => 'child3-package'}]);

=head2 get - overridden get accessor from Class::Accessor

Fetches XML over HTTP based on self->util->base_uri().

=head2 list - default handling for reading lists of objects

  my $oDOM = $oDerived->list();

=head2 read - default handling for reading individual objects

  my $oDOM = $oDerived->read();

=head2 create - default handling object creation

  my $oDOM = $oDerived->create();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for a new entity, up to the
  create_xml aspects of the service.

=head2 update - default handling object update

  my $oDOM = $oDerived->update();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for an existing entity ,up to the
  update_xml aspects of the service.

=head2 retry - method retries connecting to npg_qc before croaking with an error=head1 DIAGNOSTICS

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
npg_qc::api::base
Readonly

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
