#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-09-29
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: summary.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/api/summary.pm $
#
package npg_qc::api::summary;
use strict;
use warnings;
use Class::Std;
use Carp;
use base qw(npg_qc::api::base);
use Readonly;
use JSON;
use npg_qc::api::summary::chip;
use npg_qc::api::summary::lane;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

{
  ## no critic (ProhibitUnusedVariables)
  my %id_run_of        :ATTR( :get<id_run>,         :set<id_run>         );
  my %end_of           :ATTR( :get<end>,            :set<end>            );
  my %paired_of        :ATTR( :get<paired>,         :set<paired>         );
  my %id_run_pair_of   :ATTR( :get<id_run_pair>,    :set<id_run_pair>    );
  my %summaries_of     :ATTR( :get<summaries>,      :set<summaries>      );
  my %complete_runs_of :ATTR( :get<complete_runs>,  :set<complete_runs>  );
  my %chip_of          :ATTR( :get<chip_summary>,   :set<chip_summary>   );
  my %lanes_of         :ATTR( :get<lane_summaries>, :set<lane_summaries> );
  ## use critic

  sub fields {
    return qw(
              id_run
              end
              paired
            );
  }

  sub id_run {
    my ($self, $id_run) = @_;
    if ($id_run) {
      $self->set_id_run($id_run);
    }
    return $self->get_id_run();
  }

  sub id_run_pair {
    my ($self, $id_run_pair) = @_;
    if ($id_run_pair) {
      $self->set_id_run_pair($id_run_pair);
    }
    return $self->get_id_run_pair();
  }

  sub end {
    my ($self, $end) = @_;
    if ($end) {
      $self->set_end($end);
    }
    return $self->get_end();
  }

  sub paired {
    my ($self, $paired) = @_;
    if ($paired) {
      $self->set_paired($paired);
    }
    return $self->get_paired();
  }

  sub complete_runs {
    my ($self) = @_;

    if (!$self->get_complete_runs()) {
      my $util = $self->util();
      my $pkg        = ref $self;
      my ($obj_type) = $pkg =~ /([^:]+)$/mxs;
      my $obj_pk     = $self->primary_key();
      my $obj_uri    = sprintf '%s/%s', $util->base_uri(), $obj_type;
      my $response   = $util->get($obj_uri);

      if(!$response->is_success()) {
        croak $response->status_line();
      }

      $self->set_summaries($util->parser->parse_string($response->content()));
      my $complete_run_summaries = $self->get_summaries()->getElementsByTagName('complete_run_summaries')->[0];

      my $store = [map { $self->new_from_xml($pkg, $_) } $complete_run_summaries->getElementsByTagName('run')];
      $self->set_complete_runs($store);
    }

    return $self->get_complete_runs();
  }

  sub runs_with_fully_complete_pairs {
    my ($self) = @_;
    my $paired_runs;
    my $complete_runs;
    my $pair_one;
    my $pair_two;
    foreach my $run (@{$self->complete_runs()}) {
      if (!$run->paired()) {
        push @{$complete_runs}, $run;
      } else {
        if ($run->end == 1 && !$pair_one->{$run->id_run()}) {
          push @{$paired_runs->{$run->id_run()}}, $run;
	  $pair_one->{$run->id_run()}++;
	} elsif ($run->end == 2 && !$pair_two->{$run->id_run()}) {
          push @{$paired_runs->{$run->id_run()}}, $run;
	  $pair_two->{$run->id_run()}++;
	} else {
	}
      }
    }
    foreach my $run (sort keys %{$paired_runs}) {
      if (scalar@{$paired_runs->{$run}} == 2) {
        push @{$complete_runs}, $paired_runs->{$run}->[0];
      }
    }
    return $complete_runs;
  }

  sub read_summary {
    my ($self) = @_;
    my $util = $self->util();
    my $pkg        = ref $self;
    my ($obj_type) = $pkg =~ /([^:]+)$/mxs;
    my $obj_pk     = $self->primary_key();
    my $pk_value   = $self->$obj_pk();
    my $obj_uri    = sprintf '%s/%s/%s.json', $util->base_uri(), $obj_type, $pk_value;
    my $response   = $util->get($obj_uri);

    if(!$response->is_success()) {
      croak $response->status_line();
    }

    my $summary_hash = from_json($response->content());
    my $chip_summary = $summary_hash->{chip_summary};
    my $id_run = $summary_hash->{id_run};
    $self->id_run($id_run);
    my $lane_results_summary_read_one = $summary_hash->{lane_results_summary_read_one};
    $self->set_chip_summary($self->new_from_json($pkg.q{::chip}, $chip_summary));
    my $lane_summaries = {};
    $self->set_lane_summaries($lane_summaries);

    foreach my $lane (sort { $a <=> $b } keys %{$lane_results_summary_read_one}) {
      my $temp = $lane_results_summary_read_one->{$lane};
      $temp->{lane} = $lane;
      $temp->{id_run} = $id_run;
      push @{$lane_summaries->{read_one}}, $self->new_from_json($pkg.q{::lane}, $temp);
    }

    if ($summary_hash->{id_run_pair}) {
      my $id_run_pair = $summary_hash->{id_run_pair};
      $self->id_run_pair($id_run_pair);
      my $lane_results_summary_read_two = $summary_hash->{lane_results_summary_read_two};
      foreach my $lane (sort { $a <=> $b } keys %{$lane_results_summary_read_two}) {
        my $temp = $lane_results_summary_read_one->{$lane};
        $temp->{lane} = $lane;
        $temp->{id_run} = $id_run_pair;
        push @{$lane_summaries->{read_two}}, $self->new_from_json($pkg.q{::lane}, $temp);
      }
    }
    return 1;
  }

}
1;
__END__

=head1 NAME

npg_qc::api::summary

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - Constructor

  my $oSummary = npg_qc::api::summary->new({'util' => $oUtil,});

=head2 fields - returns an array of the fields in the run_config table

  my $aFields = $oSummary->fields();

=head2 id_run - accessor for id_run. takes an id as an optional parameter (which it stores) and returns the id_run

  my $id_run = $oSummary->id_run($id_run);

=head2 id_run_pair - accessor for id_run_pair. takes an id as an optional parameter (which it stores) and returns the id_run_pair

  my $id_run_pair = $oSummary->id_run_pair($id_run_pair);

=head2 end - accessor for end. takes an end as an optional parameter (which it stores) and returns the end

  my $end = $oSummary->end($end);

=head2 util - util instance for this resource set

  Returns a cached object if one exists, or creates and caches a new one

  my $oUtil = $oDerived->util();

=head2 complete_runs - fetches all complete runs from the npg_qc application, and turns into array of npg_qc::api::summary objects

  my $aCompleteRuns = $oSummary->complete_runs();

=head2 read_summary - fetches the summary Illumina results for a run, and stores them internally as summary::chip and summary::lane objects

  $oSummary($id_run);
  $oSummary->read_summary();

=head2 get_chip_summary - returns a summary::chip object with the chip summary information for the run

  my $oChipSummary = $oSummary->get_chip_summary();

=head2 get_lane_summaries - returns a hash, containing the key(s) read_one (and read_two if paired end run). Each of these keys contains an array (in lane order) of summary::lane objects

  my $hLaneSummaries = $oSummary->get_lane_summary();

  my $aReadOneLaneSummaries = $hLaneSummaries->{read_one};
  my $aReadTwoLaneSummaries = $hLaneSummaries->{read_two};

  my $oLaneSummaryRead1Lane1 = $aReadOneLaneSummaries->[0];

=head2 new_from_xml - create and populate an npg::api::* from a XML::LibXML::Element

  my $oObj = $oDerived->new_from_xml('npg::api::<package>', $oXMLElement);

=head2 get - overridden get accessor from Class::Accessor

Fetches XML over HTTP based on self->util->base_uri().

=head2 list - default handling for reading lists of objects

  my $oDOM = $oDerived->list();

=head2 read - default handling for reading individual objects

  my $oDOM = $oDerived->read();

=head2 create - default handling object creation

  my $oDOM = $oDerived->create();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for a new entity, up to the
  create_xml aspects of the service.

=head2 update - default handling object update

  my $oDOM = $oDerived->update();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for an existing entity ,up to the
  update_xml aspects of the service.

=head2 retry - method retries connecting to npg_qc before croaking with an error=head1 DIAGNOSTICS

=head2 paired
=head2 runs_with_fully_complete_pairs


=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
npg_qc::api::base
Readonly

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
