#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-09-04
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: Loader.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/autoqc/db/Loader.pm $
#

package npg_qc::autoqc::db::Loader;

use strict;
use warnings;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly;
use File::Spec;

use npg_qc::autoqc::db::DB;

with 'MooseX::Getopt';

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

has 'path'            => (  is          => 'ro',
                            isa         => 'ArrayRef',
                            required    => 1,
                         );

has 'id_run'          => (  is           => 'ro',
                            isa          => 'Int',
                            required     => 1,
		         );

has 'json_files'     => (  is           => 'ro',
                            isa          => 'ArrayRef',
                            required     => 0,
                            lazy_build   => 1,
                         );

sub _build_json_files {
  my $self = shift;

  my @json_files = ();
  foreach my $path (@{$self->path}) {
    my @files = glob File::Spec->catfile($path, q[*.json]);
    if (@files) {
      push @json_files, @files;
    }
  }

  return \@json_files;
}


sub BUILD {
  my $self = shift;
  if (!@{$self->path}) {
    croak q[Array of json file locations is empty];
  }
  foreach my $path (@{$self->path}) {
    if (!$path) {
      croak q[Empty string given as one of the paths];
    }
    if (!-e $path) {
      croak qq[Non-existing path $path as one of the paths];
    }
    if (!-d $path) {
      croak qq[One of the paths, $path, is not a directory];
    }
  }
  return;
}


sub process_json_files{
  my $self = shift;

  my $json_files = $self->json_files();
  my $count = 0;
  foreach my $json_file (@{$json_files}){

    print "Loading $json_file\n" or croak 'can not print';
    eval{
      npg_qc::autoqc::db::DB->save_from_jsonfile($json_file);
      $count++;
      1;
    } or do {
      if($EVAL_ERROR =~ /execute[ ]failed:[ ]Duplicate[ ]entry/mxs){
        print "data from this file already in the database\n" or carp 'can not print: data from this file already in the database';
      }else{
        croak $EVAL_ERROR;
      }
    };
  }
  print "$count files have been saved out of ". (scalar @{$json_files}) ."\n" or carp "can not print: $count files have been saved";
  return $count;
}

sub num_db_records {
  my $self = shift;

  my $checks_list = npg_qc::autoqc::autoqc->checks_list();
  my $total_lane_count_db = 0;
  foreach my $qc_check_name (@{$checks_list}){
    $total_lane_count_db += npg_qc::autoqc::db::DB->lane_count_per_run_db($qc_check_name, $self->id_run);
  }
  return $total_lane_count_db;
}

sub load{
  my $self = shift;
  if (scalar @{$self->json_files} == 0) {
     croak q[No json files found in given directories ] . ( join q[ ], @{$self->path} );
  }
  $self->process_json_files();
  return 1;
}

no Moose;
__PACKAGE__->meta->make_immutable;

1;
__END__

=head1 NAME

npg_qc::autoqc::db::Loader

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This class will glob all the json files in given directories
and load the data into the corresponding database table
based the class name inside the json file; does not update/replace
existing records.

=head1 SUBROUTINES/METHODS

=head2 path - a reference to an array with directory names from where json files have to be loaded

=head2 id_run - run id

=head2 json_files - a reference to a list of json files

=head2 process_json_files - load the data into database, using the json files in the given path

=head2 num_db_records - total number of db records for a run for all lanes

=head2 load - check data completeness in db, if not, load data from json files

=head2 BUILD - 

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Moose;
Carp;
English qw{-no_match_vars};
Readonly;
srpipe::runfolder;
npg_qc::autoqc::db::DB;

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
