#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-09-02
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: Schema.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/autoqc/db/Schema.pm $
#

package npg_qc::autoqc::db::Schema;

use warnings;
use strict;
use Fey::DBIManager::Source;
use Fey::Loader;
use Fey::ORM::Schema;
use npg_qc::autoqc::autoqc;


our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

sub load_schema {
  my ($self, $db) = @_;

  my $dsn       = $db->{dsn};
  my $user      = $db->{username};
  my $password  = $db->{password};

  if($self->Schema()){

    return $self->Schema();
  }

  my $source =
     Fey::DBIManager::Source->new( dsn => $dsn, #"DBI:mysql:database=$dbname;host=$dbhost;port=$dbport",
                                   username => $user,
                                   password => $password
     );

  my $schema = Fey::Loader->new(dbh => $source->dbh() )->make_schema();
  has_schema $schema;

  __PACKAGE__->DBIManager()->add_source($source);
  __PACKAGE__->load_tables($schema);

  return $self->Schema();
}

sub load_tables{
  my ($self, $schema) = @_;

  my @checks_list = @{npg_qc::autoqc::autoqc->checks_list()};
  # add a sub table into the table loading list
  push @checks_list, q{split_stats_coverage};

  foreach my $qc_check_name (@checks_list){

    my $class = q{npg_qc::autoqc::db::checks::} . $qc_check_name;
    eval "require $class"; ## no critic (ProhibitStringyEval RequireCheckingReturnValueOfEval)1;} 
    $class->load_table($schema);
  }

  return 1;
}

1;
__END__

=head1 NAME

npg_qc::autoqc::db::Schema

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This class defines a database schema class using Fey::ORM and Fey::Loader

=head1 SUBROUTINES/METHODS

=head2 load_schema - given a database connection, load table schema using Fey loader

=head2 load_tables - given a Fey database schema 
and load a list of tables defined in npg_qc::autoqc into table corresponding table class,
plus any sub tables.

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Fey::DBIManager::Source;
Fey::Loader;
Fey::ORM::Schema;
warnings;
strict;
npg_qc::autoqc::autoqc;

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
