#########
# Author:        jo3
# Maintainer:    $Author: ajb $
# Created:       2009-10-08
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: contamination.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/autoqc/db/checks/contamination.pm $
#

package npg_qc::autoqc::db::checks::contamination;

use strict;
use warnings;
use npg_qc::autoqc::db::Schema;
use Fey::ORM::Table;
use JSON;
use Carp;

with qw( npg_qc::autoqc::role::contamination npg_qc::autoqc::db::DB);

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

sub load_table {    ## no critic (Subroutines::RequireArgUnpacking)
    my ( $self, $schema ) = @_;

    my $table = $schema->table('contamination');

    if ( !$self->Table() ) {

        has_table $table;

        my $column = $self->Table()->column('tag_index');
        my $no_tag = $column ->default()->id();
        ($no_tag) = $no_tag =~ /(-?\d+)/sxm;

        transform 'genome_factor'
            => inflate { $_[1] ? from_json($_[1]) : undef }
            => deflate { defined $_[1] ? to_json($_[1])   : $_[1] };

        transform 'contaminant_count'
            => inflate { $_[1] ? from_json($_[1]) : undef }
            => deflate { defined $_[1] ? to_json($_[1])   : $_[1] };

        transform 'info'
            => inflate { $_[1] ? from_json($_[1] ) : undef }
            => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

        transform 'tag_index'
            => deflate { defined $_[1] ? $_[1]    : $no_tag }  #to db
            => inflate { $_[1] != $no_tag ? $_[1] : undef   }; #from db
    }
    return $self->Table();
}


1;

__END__


=head1 NAME

npg_qc::autoqc::db::checks::contamination

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This class defines a database table class using Fey::ORM, linking to autoqc
data table contamination and inheriting the autoqc result Moose class and autoqc db
DB class to use their methods.

=head1 SUBROUTINES/METHODS

=head2 load_table - load table using given schema

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Fey::ORM::Table

=item JSON

=item npg_qc::autoqc::db::DB

=item npg_qc::autoqc::db::Schema

=item npg_qc::autoqc::role::contamination

=item strict

=item warnings

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

John O'Brien, jo3@sanger.ac.uk

=head1 LICENSE AND COPYRIGHT


=cut
