#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-09-02
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: qX_yield.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/autoqc/db/checks/qX_yield.pm $
#
package npg_qc::autoqc::db::checks::qX_yield;


use strict;
use warnings;
use npg_qc::autoqc::db::Schema;
use Carp;
use Fey::ORM::Table;
use JSON;

with qw( npg_qc::autoqc::role::qX_yield  npg_qc::autoqc::db::DB );

our $VERSION   = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

sub load_table { ## no critic ( Subroutines::RequireArgUnpacking )
  my ($self, $schema) = @_;

  my $table = $schema->table('qx_yield');

  if(!$self->Table()){

     has_table $table;

     my $column = $self->Table()->column('tag_index');
     my $no_tag = $column ->default()->id();
     ($no_tag) = $no_tag =~ /(-?\d+)/sxm;

     transform 'info'
       => inflate { defined $_[1] ? from_json($_[1] ) : undef }
       => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

     transform 'tag_index'
       => deflate { defined $_[1] ? $_[1]    : $no_tag }
       => inflate { $_[1] != $no_tag ? $_[1] : undef   };
  }
  return $self->Table();
}

sub qx_yields_per_run {
  my ($self, $id_run) = @_;

  if(!$id_run){

    $id_run = $self->id_run();
  }

  return __PACKAGE__->list_per_run($id_run);
}

1;
__END__

=head1 NAME

npg_qc::autoqc::db::checks::qX_yield

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This class defines a database table class using Fey::ORM,
linking to autoqc data table qx_yield,
inheriting autoqc result Moose class and autoqc db DB class to use their methods.

=head1 SUBROUTINES/METHODS

=head2 load_table - load table using given schema

=head2 qx_yields_per_run - given id_run, return a list of qx_yield objects in arrayref

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict;
warnings;
npg_qc::autoqc::db::Schema;
Fey::ORM::Table;
npg_qc::autoqc::role::qX_yield;
npg_qc::autoqc::db::DB;

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
