#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-09-24
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: tag_decode_stats.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/autoqc/db/checks/tag_decode_stats.pm $
#
package npg_qc::autoqc::db::checks::tag_decode_stats;

use strict;
use warnings;
use npg_qc::autoqc::db::Schema;
use Fey::ORM::Table;
use JSON;

with qw( npg_qc::autoqc::role::tag_decode_stats  npg_qc::autoqc::db::DB );

our $VERSION   = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };


sub load_table{ ## no critic ( Subroutines::RequireArgUnpacking )
  my ($self, $schema) = @_;

  my $table = $schema->table('tag_decode_stats');

  if(!$self->Table()){

    has_table $table;

    my $column = $self->Table()->column('tag_index');
    my $no_tag = $column ->default()->id();
    ($no_tag) = $no_tag =~ /(-?\d+)/sxm;

    transform 'tag_code'
      => inflate { defined $_[1] ? from_json($_[1] ) : undef }
      => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

    transform 'distribution_all'
      => inflate { defined $_[1] ? from_json($_[1] ) : undef }
      => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

    transform 'distribution_good'
      => inflate { defined $_[1] ? from_json($_[1] ) : undef }
      => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

    transform 'errors_good'
      => inflate { defined $_[1] ? from_json($_[1] ) : undef }
      => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

    transform 'errors_all'
      => inflate { defined $_[1] ? from_json($_[1] ) : undef }
      => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

    transform 'info'
      => inflate { defined $_[1] ? from_json($_[1] ) : undef }
      => deflate { defined $_[1] ? to_json($_[1] )   : $_[1] };

    transform 'tag_index'
       => deflate { defined $_[1] ? $_[1]    : $no_tag }
       => inflate { $_[1] != $no_tag ? $_[1] : undef   };
  }

  return $self->Table();
}

1;
__END__

=head1 NAME

npg_qc::autoqc::db::checks::tag_decode_stats

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

This class defines a database table class using Fey::ORM,
linking to autoqc data table split_stats,
inheriting autoqc result Moose class and autoqc db DB class to use their methods.

=head1 SUBROUTINES/METHODS

=head2 load_table - load table using given schema

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict;
warnings;
npg_qc::autoqc::db::Schema;
Fey::ORM::Table;
npg_qc::autoqc::role::tag_decode_stats;
npg_qc::autoqc::db::DB;

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
