#########
# Author:        John O'Brien
# Maintainer:    $Author: ajb $
# Created:       14 April 2009
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: ref_match.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/autoqc/role/ref_match.pm $
#

package npg_qc::autoqc::role::ref_match;    ## no critic (Capitalization)

use strict;
use warnings;
use Moose::Role;

with qw(npg_qc::autoqc::role::result);

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/smx; $r; };

Readonly::Scalar my $PERCENT => 100;

sub percent_count {
    my ($self) = @_;

    my %counts = %{ $self->aligned_read_count() || {} };

    my $sample_size = $self->sample_read_count() || 0;
    return if $sample_size == 0;

    my %percentage;
    foreach my $organism ( keys %counts ) {
        my $value = $PERCENT * ( $counts{$organism} / $sample_size );

        $percentage{$organism} = sprintf '%.1f', $value;
    }

    return \%percentage;
}

sub ranked_organisms {
    my ($self) = @_;

    my %rating = %{ $self->percent_count() || {} };

    my @ranked_organisms =
        reverse sort { $rating{$a} <=> $rating{$b} } keys %rating;

    return \@ranked_organisms;
}


no Moose;

1;

__END__


=head1 NAME

    npg_qc::autoqc::role::ref_match

=head1 VERSION

    $Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 percent_count

Calculate a rating value for each organism. Right now that value is the
percentage of sampled reads aligning to the organism's reference genome.

=head2 ranked_organisms

Sort the organisms represented in the reference sequence by descending order
of their normalised contamination values. Return this list as an array
reference.

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Readonly

=item strict

=item warnings

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Author: John O'Brien E<lt>jo3@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by John O'Brien

This file is part of NPG.

NPG is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut
