#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-25
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: chip_summary.pm 13782 2011-08-01 14:48:20Z ajb $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/model/chip_summary.pm $
#

package npg_qc::model::chip_summary;
use strict;
use warnings;
use base qw(npg_qc::model);
use English qw{-no_match_vars};
use Carp;
use npg_qc::model::run_tile;
use npg_qc::model::qcal;
use Readonly;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };

Readonly our $READ_LENGTH_CALCULATOR => 1000;

__PACKAGE__->mk_accessors(__PACKAGE__->fields());

sub fields {
  return qw(
            id_chip_summary
            id_run
            paired
            chip_id
            clusters
            clusters_pf
            yield_kb
            machine
          );
}

sub init {
  my $self = shift;
  if($self->{id_run} && !$self->{id_chip_summary}) {
    my $query = q(SELECT id_chip_summary
                  FROM   chip_summary
                  WHERE  id_run = ?);
    my $ref   = [];
    eval {
      $ref = $self->util->dbh->selectall_arrayref($query, {}, $self->id_run());
      1;
    } or do {
      carp $EVAL_ERROR;
      return;
    };
    if(@{$ref}) {
      $self->{'id_chip_summary'} = $ref->[0]->[0];
    }
  }
  return 1;
}

sub cycle_count {
  my $self = shift;

  if (!$self->{cycle_count}) {

    $self->{cycle_count} = $self->get_cycle_count_from_recipe($self->{id_run}, 1);
  }

  return $self->{cycle_count};
}

sub cycle_count_2nd_end {
  my $self = shift;

  if (!$self->{cycle_count_2nd_end}) {

    $self->{cycle_count_2nd_end} = $self->get_cycle_count_from_recipe($self->{id_run}, 2);
  }

  return $self->{cycle_count_2nd_end};
}

sub read_length {
  my $self = shift;

  my $read_length1 = $self->get_read_length_from_recipe($self->{id_run}, 1);
  my $return_value = $read_length1;
  if($self->paired()){
    my $read_length2 = $self->get_read_length_from_recipe($self->{id_run}, 2);
    if($read_length2){
     $return_value .=q{, }.$read_length2;
    }
  }

  return $return_value;
}

sub qcal {
  my ($self) = @_;
  if (!$self->{qcal}) {
    $self->{qcal} = npg_qc::model::qcal->new({
      util   => $self->util(),
      id_run => $self->id_run(),
    });
  }
  return $self->{qcal};
}

sub greater_than_q_twenty {
  my $self = shift;
  return $self->qcal->greater_than_q_twenty();
}

sub greater_than_q_twentyfive {
  my $self = shift;
  return $self->qcal->greater_than_q_twentyfive();
}

sub greater_than_q_thirty {
  my $self = shift;
  return $self->qcal->greater_than_q_thirty();
}

sub qcal_by_lanes {
  my $self = shift;
  return $self->qcal->qcal_by_lanes();
}

1;
__END__
=head1 NAME

npg_qc::model::chip_summary

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

  my $oChipSummary = npg_qc::model::chip_summary->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - return array of fields, first of which is the primary key

  my $aFields = $oChipSummary->fields();

=head2 init - on creation, goes to see if a summary already exists for the id_run, if provided, and creates object with it if it is

=head2 cycle_count - returns the cycle count for the run, for the first end run if paired

  my $iCycleCount = $oChipSummary->cycle_count();
  
=head2 cycle_count_2nd_end - returns the cycle count for the second end run if paired

=head2 greater_than_q_thirty - returns the number of bases above a q30 value

  my $iGreaterThanQThirty = $oChipSummary->greater_than_q_thirty();

=head2 greater_than_q_twenty - returns the number of bases above a q20 value

  my $iGreaterThanQTwenty = $oChipSummary->greater_than_q_twenty();

=head2 greater_than_q_twentyfive - returns the number of bases above a q25 value

  my $iGreaterThanQTwentyfive = $oChipSummary->greater_than_q_twentyfive();

=head2 qcal - returns, and caches, a npq_qc::mode::qcal object

  my $oQCal = $oChipSummary->qcal();
  
=head2 qcal_by_lanes - return all qcal values by lane for this run

=head2 read_length - returns the read length obtained

  my $iReadLength = $oChipSummary->read_length();

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::model

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
