#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-10
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: util.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/lib/npg_qc/util.pm $
#
package npg_qc::util;
use strict;
use warnings;
use DBI;
use Config::IniFiles;
use Carp;
use base qw(ClearPress::util Exporter);
use LoggingDBI;
use SangerWeb;
use XML::LibXML;

our $VERSION     = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mxs; $r; };
our @EXPORT_OK   = qw($MAIL_DOMAIN);
our $MAIL_DOMAIN = q(sanger.ac.uk);

__PACKAGE__->mk_accessors(qw(useragent));

sub data_path {
  return SangerWeb->data_root() . '/prodsoft/npg_qc';
}

sub dbsection {
  return $ENV{dev} || 'live';
}

sub dbname { my $self = shift; return $self->config->val($self->dbsection(), 'dbname') || q(npg); }
sub dbhost { my $self = shift; return $self->config->val($self->dbsection(), 'dbhost') || q(localhost); }
sub dbport { my $self = shift; return $self->config->val($self->dbsection(), 'dbport') || q(3306); }
sub dbuser { my $self = shift; return $self->config->val($self->dbsection(), 'dbuser') || q(root); }
sub dbpass { my $self = shift; return $self->config->val($self->dbsection(), 'dbpass') || q(); }

sub parser {
  my $self = shift;
  $self->{parser} ||= XML::LibXML->new();
  return $self->{parser};
}

1;

__END__

=head1 NAME

npg_qc::util - A database handle and utility object

=head1 VERSION

$Revision: 13782 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 data_path - path to data directory containing config.ini and templates sub-directory

  my $sPath - $oUtil->data_path();

=head2 dbsection - 'dev' or 'live' based on environment

  my $sEnv = $oUtil->dbsection();

=head2 dbname - Accessor for configuration's dbname

  my $sDBName = $oUtil->dbname();

=head2 dbhost - Accessor for configuration's dbhost

  my $sDBHost = $oUtil->dbhost();

=head2 dbuser - Accessor for configuration's dbuser

  my $sDBUser = $oUtil->dbuser();

=head2 dbpass - Accessor for configuration's dbpass

  my $sDBPass = $oUtil->dbpass();

=head2 dbport - Accessor for configuration's dbport

  my $sDBPort = $oUtil->dbport();

=head2 dbh - A database handle for the supported database

  my $oDbh = $oUtil->dbh();

=head2 parser - creates/gets existing Lib::XML parser object

 my $oParser = $oUtil->parser();

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item DBI

=item Config::IniFiles

=item Carp

=item base

=item LoggingDBI

=item SangerWeb

=item XML::LibXML

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>, & Roger Pettett, E<lt>rmp@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
