use Carp;
use English;

use npg_qc::Schema;
use npg_qc::api::util;
use npg_qc::autoqc::db::Loader;
use npg_qc::autoqc::db::Schema;

our @TABLE_NAMES = qw/QxYield InsertSize SequenceError Adapter GcFraction/;
our $STORE = q[/nfs/sf31/fastq_store_mg8];
our $TAG_INDEX = -1;

sub _get_lanes {
  my $fname = shift;
  my $runs = {};
  open(my $fh, q[<], $fname) or die $!;
  while (my $row = <$fh>) {
    if ($row eq "\n") { next; }
    my @numbers = split q[ ], $row;
    my $id_run = $numbers[0];
    my $position = $numbers[1];
    if (exists $runs->{$id_run}) {
      push @{$runs->{$id_run}}, $position;
    } else {
      $runs->{$id_run} = [$position];
    }

  }
  close $fh;

  return $runs;
}


################  Main ##############################
my $dev = q[live];
if ($ENV{dev}) {
  $dev = $ENV{dev};
}
local $ENV{dev}=$dev;
carp 'dev = ' . $ENV{dev};

my $list_in  = q[with_refs_lanes.txt];
my $runs = _get_lanes($list_in);

#my $runs = {555 => [1, 2],};

my $util = npg_qc::api::util->new({dbh => undef, useragent => undef});
my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db ={ dsn=>"DBI:mysql:database=$dbname;host=$dbhost;port=$dbport", username=>$user, password => $password };

npg_qc::autoqc::db::Schema->load_schema($db);
my $schema = npg_qc::Schema->connect();

foreach my $id_run (sort keys %{$runs}) {
  my $qc_path = join(q[/], $STORE, $id_run, q[qc]);
  if (!-e $qc_path) {
    croak qq[Qc directory for run $id_run does not exist];
  }
  carp qq[LOADING $id_run from $qc_path];

  my $transaction = sub {
    foreach my $position (@{$runs->{$id_run}}) {
      foreach my $table (@TABLE_NAMES) {
        my $rs = $schema->resultset($table)->search(
	  {id_run => $id_run, position => $position, tag_index => $TAG_INDEX,});
        if ($rs && $rs->count) {
          $rs->next->delete; 
        }
      }
    }
  };

  $schema->txn_do($transaction);
  npg_qc::autoqc::db::Loader->new(path => [$qc_path], id_run => $id_run)->load();
}

1;
