#!/usr/bin/perl

use strict;
use warnings;
use Carp;

use npg_common::extractor::fastq qw/generate_cache/;

our $FUSE = q[/fuse/mpsafs/runs];
our $STORE = q[/nfs/sf31/fastq_store_mg8];
our $SAMPLE_SIZE = 10000;
our $CACHE_DIR_NAME = q[.npg_cache_] . $SAMPLE_SIZE;

open(my $fh, q[<], "with_refs_lanes.txt") or die $!;

my $runs = {};

while (my $row = <$fh>) {
  if ($row eq "\n") { next; }
  my @numbers = split q[ ], $row;
  my $id_run = $numbers[0];
  my $position = $numbers[1];
  if (exists $runs->{$id_run}) {
    push @{$runs->{$id_run}}, $position;
  } else {
    $runs->{$id_run} = [$position];
  }
}
close $fh;

my $tag_index = undef;

foreach my $id_run (sort keys %{$runs}) {
  my $store_path = join(q[/], $STORE, $id_run);
  my $path_in = join(q[/], $FUSE, $id_run);
  if (!-e $store_path) { 
    mkdir $store_path; 
    local $ENV{NPG_FASTQ_CACHE} = join(q[/], $store_path, $CACHE_DIR_NAME);
    carp qq[Caching run $id_run];
    foreach my $position (sort @{$runs->{$id_run}}) {
      generate_cache($path_in, $id_run, $position, $tag_index, $SAMPLE_SIZE);
    }
  }
}

1;
