#!/software/bin/perl -wT
#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-07-23
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: npg_qc_api_monitoring.pl 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/scripts/npg_qc_api_monitoring.pl $
#

use strict;
use warnings;
use lib qw(/software/solexa/lib/site_perl/);
use Carp;
use English qw{-no_match_vars};

use SangerPaths qw(core ensembl tracking trackingnpg badger);
use npg_qc::api::util;

use npg_qc::api::run_timeline;
use npg_qc::api::run_caching;
use npg_qc::api::monitoring::Runlog;
use npg_qc::api::monitoring::Runinfo;
use npg_qc::api::monitoring::Recipe;
use npg_qc::api::monitoring::Cluster_Density;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/msx; $r; };

$ENV{'PATH'} = join q(:), qw(/bin /usr/bin /usr/local/bin);

#$ENV{dev} = 'dev';

#my $uri  = q{http://deskpro19635.dynamic.sanger.ac.uk:8080/cgi-bin/prodsoft/npg_qc/npg_qc/};
my $uri  = q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg_qc/npg_qc/};

run_timeline();

run_caching();

recipe();

runinfo();

cluster_density();

runlog();

0;

sub runlog {

  print {*STDERR} qq{-----------------------\n};

  my $util = npg_qc::api::util->new({
    useragent => undef,
    base_uri  => $uri,
    parser    => undef,
  });

  my $runlog_monitor = npg_qc::api::monitoring::Runlog->new({util => $util});

  eval{
    $runlog_monitor->run();
    1;
  } or do{
      croak $EVAL_ERROR;
  };
  return 1;
}

sub recipe {

  print {*STDERR} qq{-----------------------\n};
  my $recipe_monitor = npg_qc::api::monitoring::Recipe->new();

  eval{
    $recipe_monitor->run();
    1;
  } or do{
      croak $EVAL_ERROR;
  };
  return 1;
}

sub runinfo{

  print {*STDERR} qq{-----------------------\n};

  my $runinfo_monitor = npg_qc::api::monitoring::Runinfo->new();

  eval{
    $runinfo_monitor->run();
    1;
  } or do{
      croak $EVAL_ERROR;
  };
  return 1;
}

sub cluster_density {

  print {*STDERR} qq{-----------------------\n};

  my $cluster_density_monitor = npg_qc::api::monitoring::Cluster_Density->new();

  eval{
    $cluster_density_monitor->run();
    1;
  } or do{
      croak $EVAL_ERROR;
  };
  return 1;
}

sub run_timeline{

  print {*STDERR} qq{-----------------------\nProcessing run timeline data...\n};

  my $util = npg_qc::api::util->new({
    useragent => undef,
    base_uri  => $uri,
    parser    => undef,
  });

  my $run_timeline_api = npg_qc::api::run_timeline->new({util=>$util});

  eval{
    $run_timeline_api->save_all_runs_date();
    1;
  } or do {
    croak $EVAL_ERROR;
  };
  print {*STDERR} qq{All run timeline data processed successfully.\n};
  return 1;
}

sub run_caching {

  print {*STDERR} qq{-----------------------\nRun caching...\n};

  my $util = npg_qc::api::util->new({
    useragent => undef,
    base_uri  => $uri,
    parser    => undef,
  });

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});
  eval{
    $api_run_caching->cache_all();
    1;
  } or do{
    croak $EVAL_ERROR;
  };

  print {*STDERR} "Finish run caching\n";

  return 1;
}

=head1 NAME

npg_qc_api_monitoring.pl

=head1 VERSION

$Revision: 13782 $

=head1 USAGE

  npg_qc_api_monitoring.pl

=head1 CONFIGURATION

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item npg_qc::api::util

=item SangerPaths

=item npg_qc::api::monitoring::Runlog

=item npg_qc::api::run_timeline

=item npg_qc::api::run_caching

=item npg_qc::api::monitoring::Runinfo

=item npg_qc::api::monitoring::Recipe

=item npg_qc::api::monitoring::Cluster_Density

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This file is part of NPG.

NPG is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

