#########
# Author:        mg8
# Maintainer:    $Author: ajb $
# Created:       9 August 2010
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 10-file_store.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/10-file_store.t $
#

use strict;
use warnings;
use Test::More tests => 24;
use Test::Exception;
use Perl6::Slurp;
use File::Spec;
use Moose::Meta::Class;

use npg_testing::db;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mx; $r; };



use_ok('npg_qc::file_store');

isa_ok(npg_qc::file_store->new(path => ['t']), 'npg_qc::file_store');

throws_ok {npg_qc::file_store->new()} qr/Attribute \(path\) is required /, 'error if path is not set';

{
  my $s = npg_qc::file_store->new(path => ['t']);
  is ($s->extension, q[.fastqcheck], 'default extension');
}

{
  my $s = npg_qc::file_store->new(path => ['t/data/autoqc'], extension => q[.fastq]);
  is ($s->extension, q[.fastq], 'extension set');
  is (scalar @{$s->_get_files}, 6, '6 fastq files found');

  $s->path(['t/data/autoqc', 't/data/autoqc/load']);
  $s->extension(q[.json]);
  is (scalar @{$s->_get_files}, 27, '27 .json files found in two directories');
}

{
  my $schema = Moose::Meta::Class->create_anon_class(
          roles => [qw/npg_testing::db/])
          ->new_object({})->create_test_db(q[npg_qc::Schema]);

  my $s = npg_qc::file_store->new(path => ['t/data/autoqc'], extension => q[.dodo], _schema => $schema);
  is ($s->extension, q[.dodo], 'extension set');
  is (scalar @{$s->_get_files}, 0, 'no dodo files found');
  my $num_saved;
  lives_ok {$num_saved = $s->save_files} 'saving files when there are no files lives';
  is ($num_saved, 0, 'no files are saved');
}

{
  my $schema = Moose::Meta::Class->create_anon_class(
          roles => [qw/npg_testing::db/])
          ->new_object({})->create_test_db(q[npg_qc::Schema]);

  my $path = 't/data/autoqc';
  my $s = npg_qc::file_store->new(path => [$path], _schema => $schema);
  my $num_saved;
  lives_ok {$num_saved = $s->save_files} 'saving fasqcheck files lives';

  my $expected = 2;
  is ($num_saved, $expected, qq[$expected fastqcheck files saved]);
  is ($s->_schema->resultset('Fastqcheck')->search({})->count, $expected,
                qq[$expected rows created in the table]);

  my $fname = q[wrong_max.fastqcheck];
  my $row = $s->_schema->resultset('Fastqcheck')->search({file_name => $fname,})->first;
  ok ($row, qq[row for $fname exists]);
  is ($row->file_content,
    q[9855354 sequences, 532189116 total length, 54.00 average, 55 max] . qq[\n],
    qq[$fname content saved correctly]);

  $fname = q[empty.fastqcheck];
  $row = $s->_schema->resultset('Fastqcheck')->search({file_name => $fname,})->first;
  ok ($row, qq[row for $fname exists]);
  is ($row->file_content, q[], qq[$fname saved correctly]);
}

{
  my $schema = Moose::Meta::Class->create_anon_class(
          roles => [qw/npg_testing::db/])
          ->new_object({})->create_test_db(q[npg_qc::Schema]);

  my $path = 't/data/autoqc';
  my $s = npg_qc::file_store->new(path => [$path], extension => q[.json], _schema => $schema);
  is ($s->extension, q[.json], 'extension is .json');
  my $num_saved;
  lives_ok {$num_saved = $s->save_files} 'saving json files lives';
  my $expected = 23;
  is ($num_saved, $expected, qq[$expected json files saved]);

  is ($s->_schema->resultset('Fastqcheck')->search({})->count, $expected, qq[$expected rows created in the table]);

  my $fname=  q[4453_2#0.qX_yield.json];
  my $content = slurp(File::Spec->catfile($path, $fname));

  my $row = $s->_schema->resultset('Fastqcheck')->search({file_name => $fname,})->first;
  ok ($row, qq[row for $fname exists]);
  is ($row->file_content, $content, 'file content saved correctly');
}

1;
