#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2009-01-19
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 30-api-analysis.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/30-api-analysis.t $
#

use strict;
use warnings;
use Test::More tests => 52;
use English qw(-no_match_vars);
use IO::Scalar;
use lib qw{t};
use SangerPaths qw(ensembl core badger trackingnpg tracking);
use t::api::util;
use npg::api::util;
use t::useragent;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::analysis_lane');
use_ok('npg_qc::api::analysis');

{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/analysis.xml} => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/analysis.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg/npg/'});

  my $analysis = npg_qc::api::analysis->new({util => $util});
  isa_ok($analysis, 'npg_qc::api::analysis', '$analysis');

  eval { $analysis->get_analysis_data([1,2,3,4]); };
  is($EVAL_ERROR, q{}, 'no croak running get_analysis_data([1,2,3,4])'); 

  my $analyses = $analysis->get_analyses();
  isa_ok($analyses, 'ARRAY', '$analysis->get_analyses()');
  is(scalar@{$analyses}, 4, 'Four analyses obtained');

  my $a1 = $analyses->[0];
  isa_ok($a1, 'npg_qc::api::analysis', '$analyses->[0]');
  is($a1->id_run(), 1, 'correct id_run');
  is($a1->id_analysis(), 1, 'correct id_analysis');
  is($a1->date(), '2009-01-19 12:00:00', 'correct date');
  is($a1->folder(), '/this/is/the/directory/for/run/1', 'correct folder');
  is($a1->iscurrent(), 1, 'correct value for iscurrent');

  my $lanes = $a1->get_analysis_lanes();
  isa_ok($lanes, 'ARRAY', '$a1->get_analysis_lanes()');
  is(scalar@{$lanes}, 8, 'Eight analysis_lanes obtained');

  my $al = $lanes->[4];
  isa_ok($al, 'npg_qc::api::analysis_lane', '$lanes->[4]');
  foreach my $f ($al->fields()) {
    if ($f ne 'id_run' && $f ne 'id_analysis') {
      is($al->$f(), 5, "$f method is 5");
    } else {
      is($al->$f(), 1, "$f method is 1");
    }
  }
}
1;
