use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';
use Test::Exception;
use lib qw{t};
use SangerPaths qw(core);
use t::api::util;

BEGIN {
  use_ok(q{npg_qc::api::loader::Fastqcheck});
}

my $util = t::api::util->new({
                dbh => 'dummy',
                fixtures => 1,
                useragent => undef,
                });

{
  #test for paird run
  my $loader;
  
  my $runfolder_path = qq{t/data/nfs/sf44/IL6/outgoing/100125_IL6_4308};

  lives_ok { $loader = npg_qc::api::loader::Fastqcheck->new({
                            runfolder_path => $runfolder_path,
                            util => $util,
  }); } q{loader object creation ok};

  isa_ok($loader, q{npg_qc::api::loader::Fastqcheck}, q{$loader});

  lives_ok { $loader->run(); } q{4308 fastqcheck runs ok};
}
{
  #test for phix run
  my $loader;

  $ENV{NPG_WEBSERVICE_CACHE_DIR} = q{t/data/nfs/sf44/ILorHSany_sf25/analysis/110127_IL28_05737/cache_webservice_analysis_20110204-042353};

  my $runfolder_path = q{t/data/nfs/sf44/ILorHSany_sf25/analysis/110127_IL28_05737};
  my $recalibrated_path = q{t/data/nfs/sf44/ILorHSany_sf25/analysis/110127_IL28_05737/Data/Intensities/Bustard1.8.1a2_04-02-2011_RTA.2/GERALD_04-02-2011_RTA};

  lives_ok { $loader = npg_qc::api::loader::Fastqcheck->new({
                            runfolder_path => $runfolder_path,
                            recalibrated_path => $recalibrated_path,
                            util => $util,
  }); } q{loader object creation ok};

  isa_ok($loader, q{npg_qc::api::loader::Fastqcheck}, q{$loader});

  lives_ok { $loader->run(); } q{5737 fastqcheck with-phix runs ok};
}

1;
