use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';
use Test::Exception;
use lib qw{t};
use SangerPaths qw(ensembl core badger trackingnpg tracking);
use t::api::util;
use npg::api::util;
use t::useragent;

BEGIN {
  use_ok(q{npg_qc::api::monitoring::Cluster_Density});
}

my $ua_mock = {};
my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
my $npg_api_util = npg::api::util->new({useragent => $ua});
my $util = t::api::util->new({
                npg_api_util => $npg_api_util,
                fixtures => 1,
                useragent => $ua,
                });

{
  my $monitor;
  lives_ok { $monitor = npg_qc::api::monitoring::Cluster_Density->new(); } q{loader object creation ok};

  isa_ok( $monitor, q{npg_qc::api::monitoring::Cluster_Density}, q{object test});

  isa_ok( $monitor->util(), q{npg_qc::api::util}, q{util test});

  isa_ok( $monitor->dbh(), q{DBI::db}, q{dbi test});
  
  my $runlist_db = $monitor->runlist_db();
  is(scalar keys %{$runlist_db}, 0, 'no run in the database');
 
  $monitor->pattern_prefix('t/data/nfs/sf44/IL1');
  $monitor->glob_pattern('outgoing/*/Data/reports/NumClusters\ By\ Lane.xml'); 
  is(scalar @{$monitor->runfolder_list_todo()}, 2, 'two runfolder found');
  is($monitor->runfolder_list_todo()->[0], 't/data/nfs/sf44/IL1/outgoing/080530_IL1_0956/', 'correct first runfolder');
  
  lives_ok { $monitor->save_one_run_data($monitor->runfolder_list_todo()->[1]); } q{save one run data};
  $monitor->clear_runlist_db();
  $runlist_db = $monitor->runlist_db();
  is(scalar keys %{$runlist_db}, 1, '1 run in the database now');
  
  lives_ok { $monitor->run();} q{save all the runs not in the database};
  
  $monitor->clear_runlist_db();
  $runlist_db = $monitor->runlist_db();
  is(scalar keys %{$runlist_db}, 1, '1 run in the database now');
}

1;
