use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';
use Test::Exception;
use lib qw{t};
use SangerPaths qw(core);
use t::api::util;
use t::useragent;

BEGIN {
  use_ok(q{npg_qc::api::monitoring::Recipe});
}

my $ua_mock = {};
my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
my $npg_api_util = npg::api::util->new({useragent => $ua});
my $util = t::api::util->new({
                npg_api_util => $npg_api_util,
                fixtures => 1,
                useragent => $ua,
                });
{

  my $recipe_monitor;

  lives_ok { $recipe_monitor = npg_qc::api::monitoring::Recipe->new({
                            pattern_prefix  => 't/data/nfs/sf44/IL*',
                            glob_pattern    => q{*/*4308/Recipe*.xml},
                            util => $util,
  }); } q{loader object creation ok};

  isa_ok($recipe_monitor, q{npg_qc::api::monitoring::Recipe}, q{$recipe_monitor});
  
  is(scalar keys %{$recipe_monitor->runlist_db()}, 5, 'correct number of runs in db');
  is(scalar @{$recipe_monitor->runfolder_list_todo()}, 1, 'correct number of runs to do');

  lives_ok { $recipe_monitor->run(); } q{recipe loading ok};
}

1;
