#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-07-23
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 30-api-run.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/30-api-run.t $
#

use strict;
use warnings;
use Test::More tests => 66;
use English qw(-no_match_vars);
use IO::Scalar;
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use npg::api::util;
use t::useragent;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::run');
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//main.json}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/main.json},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  my $run = npg_qc::api::run->new({util => $util});

  my $all_runs;
  eval { $all_runs = $run->all_runs(); };
  is($EVAL_ERROR, q{}, 'no croak on fetching run list ($run->all_runs())');
  is($run->all_runs(), $all_runs, 'run list cached ok');
  isa_ok($all_runs, 'ARRAY', '$run->all_runs()');
  is(scalar@{$all_runs}, 671, '671 run ids found');
}

{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//run_tile.json?id_run=1720&runpair=1}  => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_tile_list_json_id_run_1720.json},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//summary/1720.json}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/summary_1682.json},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  my $run = npg_qc::api::run->new({util => $util});
  isa_ok($run, 'npg_qc::api::run', '$run');
  my @fields = $run->fields();
  is($fields[0], 'id_run', '$run->fields()[0] is id_run');
  is(scalar@fields, 3, 'fields array is length 3');
  $run->id_run(1720);
  eval { $run->read_run_tiles(); };
  is($EVAL_ERROR, q{}, 'no croak on read_run_tiles');
  is($run->get_id_run(), 1720, 'get_id_run is correct');
  is($run->get_is_paired(), 1, 'get_is_paired is correct');
  is($run->get_id_run_pair(), 1738, 'get_id_run_pair is correct');
  my $read_one_lanes = $run->get_read_one_lanes();
  isa_ok($read_one_lanes, 'ARRAY', '$run->get_read_one_lanes()');
  is(scalar@{$read_one_lanes}, 8, '8 lanes');
  my $first_lane = $read_one_lanes->[0];
  isa_ok($first_lane, 'npg_qc::api::run::lane', '$run->get_read_one_lanes->[0]');
  @fields = $first_lane->fields();
  is(scalar@fields, 1, 'fields array length is 1');
  is($first_lane->get_lane(), 1, '$first_lane->get_lane() is 1');
  is($first_lane->get_read_end(), 1, '$first_lane->get_read_end() is correct');
  my $tiles = $first_lane->get_tiles();
  isa_ok($tiles, 'ARRAY', '$first_lane->get_tiles()');
  is(scalar@{$tiles}, 100, '100 tiles');
  my $first_tile = $tiles->[0];
  isa_ok($first_tile, 'npg_qc::api::run::lane::tile');
  is($first_tile->get_tile(), 1, '$first_tile->get_tile() is 1');
  is($first_tile->get_row(), 0, '$first_tile->get_row() is 0');
  is($first_tile->get_col(), 0, '$first_tile->get_col() is 0');
  is($first_tile->get_avg_newz(), 3565, '$first_tile->get_avg_newz() is 0');
  my $tile_qc_data = $first_tile->get_tile_qc_data();
  isa_ok($tile_qc_data, 'HASH', '$first_tile->get_tile_qc_data()');
  is($first_lane->id_run(), 1720, 'id_run can be obtained from lane object');
  is($first_tile->id_run(), 1720, 'id_run can be obtained from tile object');
  is($first_tile->get_lane(), 1, 'lane can be obtained from tile object');

  my $read_two_lanes = $run->get_read_two_lanes();
  isa_ok($read_two_lanes, 'ARRAY', '$run->get_read_two_lanes()');
  is(scalar@{$read_two_lanes}, 8, '8 lanes');

  my $tile = $run->get_read_one_lanes->[3]->get_tiles->[25];
  @fields = $tile->fields();
  is(scalar@fields, 6, 'fields array length is 6');
  is($tile->get_id_run_tile, 1363041, 'id_run_tile is correct');
  is($tile->id_run(), 1720, 'id_run is correct');
  is($tile->get_lane(), 4, 'lane is correct');
  is($tile->get_tile(), 26, 'tile is correct');
  is($tile->get_row(), 25, 'row is correct');
  is($tile->get_col(), 0, 'col is correct');
  is($tile->get_avg_newz(), '-2665.72', 'avg_newz is correct');
  is($tile->get_read_end(), 1, 'read_end is correct');
  $tile_qc_data = $tile->get_tile_qc_data();
  is($tile_qc_data->{clusters_raw}, 190260, 'clusters_raw is correct');
  is($tile_qc_data->{pf_cluster_count}, 64669, 'pf_cluster_count is correct');
  is($tile_qc_data->{pc_pf_cluster}, 33.99, 'pc_pf_cluster is correct');
  is($tile_qc_data->{av_1st_cycle_int_pf}, 241.05, 'av_1st_cycle_int_pf is correct');
  is($tile_qc_data->{av_pc_int_after_20_cycles_pf}, 49.26, 'av_pc_int_after_20_cycles_pf is correct');
  is($tile_qc_data->{pc_align_pf}, '95.60', 'pc_align_pf is correct');
  is($tile_qc_data->{av_align_score_pf}, '44.10','av_align_score_pf is correct');
  is($tile_qc_data->{pc_err_rate_pf}, 1.05, 'pc_err_rate_pf is correct');

  $tile = $run->get_read_two_lanes->[6]->get_tiles->[76];
  is($tile->get_id_run_tile, 1393126, 'id_run_tile is correct');
  is($tile->id_run(), 1738, 'id_run is correct');
  is($tile->get_lane(), 7, 'lane is correct');
  is($tile->get_tile(), 77, 'tile is correct');
  is($tile->get_row(), 23, 'row is correct');
  is($tile->get_col(), 1, 'col is correct');
  is($tile->get_avg_newz(), '1758.86', 'avg_newz is correct');
  is($tile->get_read_end(), 2, 'read_end is correct');
  $tile_qc_data = $tile->get_tile_qc_data();
  is($tile_qc_data->{clusters_raw}, 197670, 'clusters_raw is correct');
  is($tile_qc_data->{pf_cluster_count}, 82626, 'pf_cluster_count is correct');
  is($tile_qc_data->{pc_pf_cluster}, '41.80', 'pc_pf_cluster is correct');
  is($tile_qc_data->{av_1st_cycle_int_pf}, 185.92, 'av_1st_cycle_int_pf is correct');
  is($tile_qc_data->{av_pc_int_after_20_cycles_pf}, 78.88, 'av_pc_int_after_20_cycles_pf is correct');
  is($tile_qc_data->{pc_align_pf}, undef, 'pc_align_pf is correct');
  is($tile_qc_data->{av_align_score_pf}, undef,'av_align_score_pf is correct');
  is($tile_qc_data->{pc_err_rate_pf}, undef, 'pc_err_rate_pf is correct');

  my $summary_obj = $run->get_summary();
  isa_ok($summary_obj, 'npg_qc::api::summary', '$run->get_summary()');
  is($summary_obj->id_run(), 1682, 'id_run has been obtained correctly from the summary JSON');
}
1;
