#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-09-29
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 30-api-summary.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/30-api-summary.t $
#

use strict;
use warnings;
use Test::More tests => 72;
use English qw(-no_match_vars);
use IO::Scalar;
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use npg::api::util;
use t::useragent;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::summary');
{
  my $ua_mock = {};

  my $ua   = t::useragent->new({is_success => 0, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  my $summary = npg_qc::api::summary->new({util => $util});
  isa_ok($summary, 'npg_qc::api::summary', '$summary');
  my $summaries;
  eval { $summaries = $summary->complete_runs(); };
  like($EVAL_ERROR, qr{error\ in\ t::useragent}, 'croaked if no success obtaining run_configs via LWP');
  eval { $summaries = $summary->read_summary(); };
  like($EVAL_ERROR, qr{error\ in\ t::useragent}, 'croaked if no success obtaining a run summary via LWP');
}
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//summary}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/completed_run_summary.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  my $summary = npg_qc::api::summary->new({util => $util});
  isa_ok($summary, 'npg_qc::api::summary', '$summary');
  my @fields = $summary->fields();
  is($fields[0], 'id_run', '$summary->fields()[0] is id_run');
  my $summaries;
  eval { $summaries = $summary->complete_runs(); };
  is($EVAL_ERROR, q{}, 'no croak on obtaining complete_run_summaries');
  isa_ok($summaries, 'ARRAY', '$summaries');
  is(scalar@{$summaries}, 8, '8 runs obtained');
  isa_ok($summaries->[0], 'npg_qc::api::summary', '$summaries->[0]');
  is($summaries->[0]->id_run(), 1, '$summaries->[0]->id_run() is ok');
  is($summaries->[0]->end(), 1, '$summaries->[0]->end() is ok');
  is($summaries->[0]->paired(), undef, '$summaries->[0]->paired() is ok');
  is($summaries->[-1]->id_run(), 956, '$summaries->[-1]->id_run() is ok');
  is($summaries->[-1]->end(), 1, '$summaries->[-1]->end() is ok');
  is($summaries->[-1]->paired(), 1, '$summaries->[-1]->paired() is ok');

  is($summary->complete_runs(), $summaries, 'complete runs cached within object');

  my $fully_complete_summaries;
  eval { $fully_complete_summaries = $summary->runs_with_fully_complete_pairs(); };
  is($EVAL_ERROR, q{}, 'no croak on obtaining fully_complete_run_summaries');
  isa_ok($fully_complete_summaries, 'ARRAY', '$fully_complete_summaries');
  is(scalar@{$fully_complete_summaries}, 4, '4 runs obtained');
  isa_ok($fully_complete_summaries->[0], 'npg_qc::api::summary', '$fully_complete_summaries->[0]');
  is($fully_complete_summaries->[-1]->id_run(), 4, '$fully_complete_summaries->[-1]->id_run() is ok');
  is($fully_complete_summaries->[-1]->end(), 1, '$fully_complete_summaries->[-1]->end() is ok');

}
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//summary/1682.json}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/summary_1682.json},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  my $summary = npg_qc::api::summary->new({util => $util});
  $summary->id_run(1682);

  eval { $summary->read_summary(); };
  is($EVAL_ERROR, q{}, 'no croak on read_summary');

  is($summary->id_run(), 1682, 'id_run is correct');
  is($summary->id_run_pair(), 1706, 'id_run_pair is correct');

  my $chip_summary = $summary->get_chip_summary();
  isa_ok($chip_summary, 'npg_qc::api::summary::chip', '$summary->get_chip_summary()');

  is($chip_summary->get_id_run(), 1682, '$chip_summary->get_id_run is correct');
  is($chip_summary->get_machine(), 'IL13', '$chip_summary->get_machine is correct');
  is($chip_summary->get_is_paired(), 1, '$chip_summary->get_is_paired is correct');
  is($chip_summary->get_id_run_pair(), 1706, '$chip_summary->get_id_run_pair is correct');
  is($chip_summary->get_cycle_count_each_run(), 37, '$chip_summary->get_cycle_count_each_run is correct');
  is($chip_summary->get_cycle_count_total(), 74, '$chip_summary->get_cycle_count_total is correct');
  is($chip_summary->get_chip_id(), 'unknown', '$chip_summary->get_chip_id is correct');
  is($chip_summary->get_clusters(), 292022202, '$chip_summary->get_clusters is correct');
  is($chip_summary->get_clusters_pf(), 138668930, '$chip_summary->get_clusters_pf is correct');
  is($chip_summary->get_yield_kb(), 5130750, '$chip_summary->get_yield_kb is correct');
  is($chip_summary->get_read_length(), 37, '$chip_summary->get_read_length is correct');
  is($chip_summary->get_pf_gt_20(), 0, '$chip_summary->get_pf_gt_20 is correct');
  is($chip_summary->get_pf_gt_25(), 0, '$chip_summary->get_pf_gt_25 is correct');
  is($chip_summary->get_pf_gt_30(), 0, '$chip_summary->get_pf_gt_30 is correct');

  my $reads = $summary->get_lane_summaries();
  isa_ok($reads, 'HASH', '$summary->get_lane_summaries()');
  is(scalar keys%{$reads}, 2, '2 keys');
  my @keys = sort keys%{$reads};
  is($keys[0], 'read_one', 'key read_one');
  is($keys[1], 'read_two', 'key read_two');
  is(scalar@{$reads->{read_one}}, 8, '8 lanes obtained for read one');
  is(scalar@{$reads->{read_two}}, 8, '8 lanes obtained for read two');
  my $lane_summary = $reads->{read_one}->[0];
  isa_ok($lane_summary, 'npg_qc::api::summary::lane', '$summary->get_lane_summaries()->[0]');
  is($lane_summary->get_id_run(), 1682, '$lane_summary->get_id_run is correct');
  is($lane_summary->get_lane(), 1, '$lane_summary->get_lane is correct');
  is($lane_summary->get_yield(), 300201, '$lane_summary->get_yield is correct');
  is($lane_summary->get_total_raw_reads(), 18301830, '$lane_summary->get_total_raw_reads is correct');
  is($lane_summary->get_total_pf_reads(), 8113550, '$lane_summary->get_total_pf_reads is correct');
  is($lane_summary->get_clusters_raw(), 183018, '$lane_summary->get_clusters_raw is correct');
  is($lane_summary->get_clusters_raw_sd(), 2799, '$lane_summary->get_clusters_raw_sd is correct');
  is($lane_summary->get_clusters_pf(), 81136, '$lane_summary->get_clusters_pf is correct');
  is($lane_summary->get_clusters_pf_sd(), 22415, '$lane_summary->get_clusters_pf_sd is correct');
  is($lane_summary->get_first_cycle_int(), 319, '$lane_summary->get_first_cycle_int is correct');
  is($lane_summary->get_first_cycle_int_sd(), 89, '$lane_summary->get_first_cycle_int_sd is correct');
  is($lane_summary->get_perc_int_20_cycles(), 69.76, '$lane_summary->get_perc_int_20_cycles is correct');
  is($lane_summary->get_perc_int_20_cycles_sd(), '19.20', '$lane_summary->get_perc_int_20_cycles_sd is correct');
  is($lane_summary->get_perc_pf_clusters(), '44.40', '$lane_summary->get_perc_pf_clusters is correct');
  is($lane_summary->get_perc_pf_clusters_sd(), 12.28, '$lane_summary->get_perc_pf_clusters_sd is correct');
  is($lane_summary->get_perc_pf_align(), '0.00', '$lane_summary->get_perc_pf_align is correct');
  is($lane_summary->get_perc_pf_align_sd(), '0.00', '$lane_summary->get_perc_pf_align_sd is correct');
  is($lane_summary->get_align_score(), '0.00', '$lane_summary->get_align_score is correct');
  is($lane_summary->get_align_score_sd(), '0.00', '$lane_summary->get_align_score_sd is correct');
  is($lane_summary->get_perc_error_rate(), '0.00', '$lane_summary->get_perc_error_rate is correct');
  is($lane_summary->get_perc_error_rate_sd(), '0.00', '$lane_summary->get_perc_error_rate_sd is correct');

}
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//summary/1706.json}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/summary_1682.json},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});
  my $summary = npg_qc::api::summary->new({util => $util});
  $summary->id_run(1706);

  eval { $summary->read_summary(); };
  is($EVAL_ERROR, q{}, 'no croak on read_summary');

  is($summary->id_run(), 1682, 'id_run is correct, swapped as used id_run_pair');
  is($summary->id_run_pair(), 1706, 'id_run_pair is correct, swapped as used id_run_pair');
}
1;
