#########
# Author:        jo3
# Maintainer:    $Author: ajb $
# Created:       30 July 2009
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 60-autoqc-checks-gc_bias.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/60-autoqc-checks-gc_bias.t $
#

use strict;
use warnings;
use Cwd;
use Test::More tests => 12;
use Test::Exception;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/msx; $r; };

my $repos = cwd . '/t/data/autoqc';

use_ok('npg_qc::autoqc::checks::gc_bias');

{
   my $check = npg_qc::autoqc::checks::gc_bias->new(
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      position  => 2,
                                                      id_run => 2549,
                                                      repository => $repos,
                                                    );
   is($check->bam_file, 't/data/autoqc/090721_IL29_2549/data/2549_2.bam', 'bam file path for lane 2');
   is($check->read_length, 23, 'read length');
}

{
   my $check = npg_qc::autoqc::checks::gc_bias->new(
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      position  => 6,
                                                      id_run    => 2549,
                                                      tag_index => 1,
                                                      repository => $repos,
                                                   );
   is($check->bam_file, 't/data/autoqc/090721_IL29_2549/data/2549_6#1.bam', 'bam file path for lane 6 tag 1');
   is($check->read_length, undef, 'read length undef for an empty fastq file');
   $check->execute;
   is ($check->result->comments, q[read length not found], 'comment when read length not found');
}

{
   my $check = npg_qc::autoqc::checks::gc_bias->new(
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      position  => 6,
                                                      id_run    => 2549,
                                                      tag_index => 1,
                                                      read_length => 0,
                                                      repository => $repos,
                                                   );
   $check->execute;
   is ($check->result->comments, q[read length negative or zero: 0], 'comment when read length is zero');
}

{
   my $check = npg_qc::autoqc::checks::gc_bias->new(
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      position  => 6,
                                                      id_run    => 2549,
                                                      tag_index => 2,
                                                      repository => $repos,
                                                   );
   is($check->bam_file, 't/data/autoqc/090721_IL29_2549/data/2549_6#2.bam', 'bam file path for lane 6 tag 2');
   throws_ok {$check->read_length} qr/No\ fastq\ files\ found/, 'error if no fastq file';
   throws_ok {$check->execute} qr/No\ fastq\ files\ found/, 'error in execute if no fastq file';
}

{
   my $check = npg_qc::autoqc::checks::gc_bias->new(
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      position  => 4,
                                                      id_run    => 2549,
                                                      repository => $repos,
                                                   );
   is($check->bam_file, 't/data/autoqc/090721_IL29_2549/data/2549_4_nonhuman.bam', 'bam file path for lane 4 non-human');
}

{
   my $check = npg_qc::autoqc::checks::gc_bias->new(
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      position  => 6,
                                                      id_run    => 2549,
                                                      tag_index => 3,
                                                      repository => $repos,
                                                   );
   is($check->bam_file, 't/data/autoqc/090721_IL29_2549/data/2549_6_nonhuman#3.bam', 'bam file path for lane 6 tag 3 non-human');
}

1;
