#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       10 November 2009
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 60-autoqc-checks-sequence_error.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/60-autoqc-checks-sequence_error.t $
#

use strict;
use warnings;
use English qw(-no_match_vars);
use File::Copy;
use Test::More tests => 49;
use Carp;
use Test::Exception;
use Test::Deep;
use File::Temp qw/ tempdir /;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/msx; $r; };

my $repos = q[t/data/autoqc];

use_ok('npg_qc::autoqc::checks::sequence_error');

{
  my @sequences;
  my @qualities;
  open ( my $fh, q{<}, q{t/data/autoqc/sequence_error/6376_1_1.fastq.10000} ) || croak q{unable to open t/data/autoqc/1937_1_1.fastq};
  my $count = 0;
  while ( <$fh> ) {
    chomp;
    if ( $count == 1 ) {
      push @sequences, $_;
    }
    if ( $count == 3 ) {
      push @qualities, $_;
      $count = 0;
    } else {
      $count++;
    }
  }
  close $fh;
  my $quality_structure = {
    15 => [],
    30 => [],
    31 => [],
  };
  my $match_array = [qw{1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 1 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 1 0 0 1 0 1 1 0 1 1 1 1 1 1 1 0 1 1 0 0 0 0 0 1 1 0 1 1 1 1 1 1 } ];
  my $index = 0;
  foreach my $q_string ( @qualities ) {
    npg_qc::autoqc::checks::sequence_error->_collate_qualities({
      match_array => $match_array,
      quality_struct => $quality_structure,
      quality_string => $q_string,
      sequence => $sequences[$index],
      flag => 0,
    });
    $index++;
  }

  my $expected_qual_struct = {
   '15' => [1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,28,0,0,0,0,0,0,43,42,48,51,56,47,0,0,0,0,0,0,0,0,0,103,0,0,0,358,0,155,282,249,402,414,392,456,485,204,189,0,0,0,0,0,0,0,0,0,0,0,268,0,0,359,0,344,340,0,293,321,320,318,349,324,342,0,366,443,0,0,0,0,0,509,547,0,589,705,681,678,649,673],
   '30' => [161,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,243,0,0,0,0,0,0,331,338,343,368,321,332,0,0,0,0,0,0,0,0,0,498,0,0,0,6486,0,1865,6913,7057,7107,7252,7368,7381,7420,624,643,0,0,0,0,0,0,0,0,0,0,0,828,0,0,970,0,1052,1103,0,1009,1054,1074,1105,967,1059,1142,0,1185,1158,0,0,0,0,0,1461,1518,0,1559,1611,1787,1857,1854,2444],
   '31' => [9808,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9729,0,0,0,0,0,0,9626,9620,9608,9581,9622,9621,0,0,0,0,0,0,0,0,0,9396,0,0,0,3155,0,2877,2805,2694,2491,2333,2239,2163,2095,9172,9168,0,0,0,0,0,0,0,0,0,0,0,8889,0,0,8656,0,8591,8540,0,8681,8607,8589,8559,8666,8600,8499,0,8432,8382,0,0,0,0,0,8013,7918,0,7837,7670,7517,7451,7482,6865],
  };
  is_deeply( $quality_structure, $expected_qual_struct, q{quality structure obtained ok} );

  foreach my $q_val ( sort { $a <=> $b } keys %{$quality_structure} ) {
    is( $quality_structure->{$q_val}[46], 0, qq{No N's give quality value at cycle 47 for qval $q_val} );
  }

  my $n_count_array = [];
  foreach my $seq ( @sequences ) {
    npg_qc::autoqc::checks::sequence_error->_collate_uncalled( {
      match_array => $match_array,
      sequence => $seq,
      count_array => $n_count_array,
      flag => 0,
    } );
  }
  my $expected = [
  30,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,1,undef,1,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,3,undef,undef,undef,1,10_000,5103,undef,undef,undef,1,1,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,15,undef,undef,15,undef,13,17,undef,17,18,17,18,18,17,17,undef,17,17,undef,undef,undef,undef,undef,17,17,undef,15,14,15,14,15,18
  ];

diag explain $n_count_array;

  is_deeply( $n_count_array, $expected, q{n_count array generated correctly} );
  is( $n_count_array->[46], 10_000, q{cycle 47 is all N's} );

}

{
  my $match_array = npg_qc::autoqc::checks::sequence_error->parsing_md_string('12A0^AA24');

  is(scalar @{$match_array}, 37, 'correct length of read');
  ok($match_array->[12], 'mismatch on position 13');
  ok(!$match_array->[36], 'match on last position');
}

{
  my $match_array = npg_qc::autoqc::checks::sequence_error->parsing_md_string('36');
  is(scalar @{$match_array}, 36, 'correct length of read');
  ok(!$match_array->[12], 'match on position 13');
  ok(!$match_array->[35], 'match on last position');
}

{
  my $match_array = npg_qc::autoqc::checks::sequence_error->parsing_md_string('0A36');
  is(scalar @{$match_array}, 37, 'correct length of read');
  ok($match_array->[0], 'mismatch on position 1');
  ok(!$match_array->[36], 'match on last position');
}

{
  my $match_array1 = npg_qc::autoqc::checks::sequence_error->parsing_md_string('35^AA1');
  my $match_array2 = npg_qc::autoqc::checks::sequence_error->modify_match_by_cigar('28M1I7M2D1M', $match_array1 );
  is (scalar @{$match_array1}, 36, 'correct array length before modifying');
  is(scalar @{$match_array2}, 37, 'correct array length after modifying');
}

{
  my $match_array1 = npg_qc::autoqc::checks::sequence_error->parsing_md_string('32A2');
  print "@{$match_array1}\n";
  my ($match_array2, $count) = npg_qc::autoqc::checks::sequence_error->modify_match_by_cigar('20M2I15M', $match_array1 );
  print "@{$match_array2}\n";
  is (scalar @{$match_array1}, 35, 'correct array length before modifying');
  is(scalar @{$match_array2}, 37, 'correct array length after modifying');
  ok($match_array1->[32], 'position 33 mismatch before modifying');
  ok(!$match_array2->[32], 'position 33 match after modifying');
}
{
  my $reverse = npg_qc::autoqc::checks::sequence_error->check_read_orientation(16);
  ok($reverse, 'reverse for 16');
}

{
  my $reverse = npg_qc::autoqc::checks::sequence_error->check_read_orientation(8);
  ok(!$reverse, 'forward for 8');
}

{
  my $reverse = npg_qc::autoqc::checks::sequence_error->check_read_orientation(128);
  ok (!$reverse, 'forward for 128');;
}

{
  my $reverse = npg_qc::autoqc::checks::sequence_error->check_read_orientation(24);
  ok ($reverse, 'reverse for 24');
}

{
  my ($match_array, $count) = npg_qc::autoqc::checks::sequence_error->matches_per_base('32A3', '2M1I34M', 0);

  is (scalar @{$match_array}, 37, 'correct array length');
  ok($match_array->[33], 'position 34 mismatch');
  ok(!$match_array->[34], 'position 35 match');
}

{
  my ($match_array, $count) = npg_qc::autoqc::checks::sequence_error->matches_per_base('34^AG2 ', '34M2D2M1S', 1);

  is (scalar @{$match_array}, 37, 'correct array length');
  ok(!$match_array->[1], 'last position 37 not mismatch, after reversing position 2');
}

{
  my $results_hash = npg_qc::autoqc::checks::sequence_error->parsing_sam('t/data/autoqc/2638_8_2.sam');
  is( $results_hash->{error_by_base}->[0], 1, 'first base error rate correct' );
  is( $results_hash->{error_by_base}->[36], 0, 'last base error rate correct' );
  is( $results_hash->{num_reads_aligned}, 43, 'correct number aligned reads' );
  is( $results_hash->{num_reads_not_aligned}, 6, 'correct number not aligned reads');
}

{
  my $error_check = npg_qc::autoqc::checks::sequence_error->new(position => 2, path => 't/data/autoqc/090721_IL29_2549/data', id_run => 2549, aligner_cmd => 'bwa', aligner_options => '-l 32 -k 2', reference=>'/nfs/repository/d0031/references/Homo_sapiens/NCBI36/all/bwa/Homo_sapiens.NCBI36.48.dna.all.fa', repository => $repos, );
  is( $error_check->generate_filename(q[fastq],2)->[0], '2549_2_2.fastq', 'generate correct name for fastq file end 2');
  is( $error_check->generate_filename(q[fastq])->[0], '2549_2.fastq', 'single end fastq file');
  is( $error_check->sample_size, 10000, 'default required sample size');
  my @fastq_files = $error_check->get_input_files ();
  is( $fastq_files[0], 't/data/autoqc/090721_IL29_2549/data/2549_2_1.fastq', 'correct first fastq full name');
}

{
  my $dir = tempdir( CLEANUP => 1 );
  my $f = $dir . q[/2549_8_1.fastq];
  `touch $f`;
  $f = $dir . q[/2549_8_2.fastq];
  `touch $f`;

  my $error_check = npg_qc::autoqc::checks::sequence_error->new(position => 8, path => $dir, id_run => 2549, aligner_cmd => 'bwa', aligner_options => '-l 32 -k 2', reference=>'/nfs/repository/d0031/references/Homo_sapiens/NCBI36/all/bwa/Homo_sapiens.NCBI36.48.dna.all.fa', repository => $repos,);
  is( $error_check->_actual_sample_size, undef, 'actual sample size undefined');
  lives_ok { $error_check->execute } 'does not fail with empty files';
  is( $error_check->_actual_sample_size, undef, 'actual sample size undefined');
  is( $error_check->result->sample_size, undef, 'result sample size undefined');
  is( $error_check->result->reference, undef, 'result reference undefined');

  `cp t/data/autoqc/090721_IL29_2549/data/2549_7_1.fastqcheck $dir`;
  $f = $dir . q[/2549_7_1.fastq];
  `touch $f`;
  $error_check = npg_qc::autoqc::checks::sequence_error->new(position => 7, path => $dir, id_run => 2549, aligner_cmd => 'bwa', aligner_options => '-l 32 -k 2', reference=>'/nfs/repository/d0031/references/Homo_sapiens/NCBI36/all/bwa/Homo_sapiens.NCBI36.48.dna.all.fa', repository => $repos,);
  is( $error_check->_actual_sample_size, undef, 'actual sample size undefined');
  lives_ok { $error_check->execute } 'does not fail with empty files';
  is( $error_check->_actual_sample_size, undef, 'actual sample size undefined');
  is( $error_check->result->sample_size, undef, 'result sample size undefined');
  is( $error_check->result->reference, undef, 'result reference undefined');
}


1;
