#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2010-06-24
# Last Modified: $Date: 2010-05-24 17:11:58 +0100 (Mon, 24 May 2010) $
# Id:            $Id: 60-autoqc-db-checks-tag_decode_stats.t 9415 2010-05-24 16:11:58Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/branches/prerelease-30.0/t/60-autoqc-db-checks-tag_decode_stats.t $
#

use strict;
use warnings;
use Test::More tests => 13;
use Test::Exception;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use t::useragent;
use Carp;

our $VERSION = do { my ($r) = q$Revision: 9415 $ =~ /(\d+)/mx; $r; };

$ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::checks::bam_flagstats');

my $util = t::api::util->new({fixtures => 1, dbh => undef, useragent => undef});

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db ={ dsn=>"DBI:mysql:database=$dbname;host=$dbhost;port=$dbport", username=>$user, password => $password };

npg_qc::autoqc::db::Schema->load_schema($db);

{
  my $stats = npg_qc::autoqc::db::checks::bam_flagstats->new( id_bam_flagstats => 1, );

  my @attributes = $stats->meta->get_all_attributes();

  is(scalar @attributes, 26, 'correct number of attributes');
  is($stats->id_run(), 4500, 'correct id run');
  is($stats->position, 1, 'correct position');
  is($stats->paired_read_duplicates, 15, 'correct paired read duplicates');
  is($stats->histogram->{1}, 1505, 'correct value for histogram bin 1');
  is($stats->percent_properly_paired, 0.0454545454545455, 'correct percent properly paired');

  is($stats->class_name(), 'bam_flagstats', 'correct name');
  is($stats->check_name(), 'bam flagstats', 'correct check name');
}

{
  my $stats;
  lives_ok {
    $stats = npg_qc::autoqc::db::checks::bam_flagstats->new(id_run => 4500, position =>1, tag_index => -1,)
  } 'no croak when query database using id_run 3500 position 1';
  is($stats->id_bam_flagstats(), 1, 'correct id for bam flagstats of run 4500 position 1');
  is($stats->tag_index(), undef, 'tag index undefined');
}

{
  my $stats = npg_qc::autoqc::db::checks::bam_flagstats->save_from_jsonfile("t/data/autoqc/4783_6_bam_flagstats.json");
  is($stats->id_bam_flagstats(), 2, 'data saved from the json file and the primary key is 2');
}

1;
