#########
# Author:        jo3
# Maintainer:    $Author: ajb $
# Created:       2008-09-02
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 60-autoqc-db-checks-contamination.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/60-autoqc-db-checks-contamination.t $
#

use strict;
use warnings;
use Test::More tests => 17;
use Test::Exception;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw( trackingnpg core );
use t::api::util;
use Carp;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/msx; $r; };

local $ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::checks::contamination');


my $util = t::api::util->new( { fixtures  => 1,
                                dbh       => undef,
                                useragent => undef, }
);

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db = { dsn      => "DBI:mysql:database=$dbname;host=$dbhost;port=$dbport",
           username => $user,
           password => $password, };

npg_qc::autoqc::db::Schema->load_schema($db);

my $contam =
    npg_qc::autoqc::db::checks::contamination->new( id_contamination => 1 );

my @attributes = $contam->meta->get_all_attributes();

# Uncomment for debugging
#foreach my $attr (@attributes) {
#    my $name = $attr->name();
#    print $name, "\t" || carp;
#    print $attr->type_constraint, "\t" || carp;
#    print $contam->$name(), "\n" || carp;
#}

is( scalar @attributes,    17, 'Correct number of attributes' );
is( $contam->id_run(),   3300, 'Correct id run' );
is( $contam->position(),    7, 'Correct position' );
is( $contam->aligner_version(), 'Bowtie 0.13.4', 'Correct aligner version' );
is( $contam->reference_version(), 'Composite_20091028',
    'Correct reference version' );
is( $contam->path(),               '/tmp', 'Correct path' );
is( $contam->pass(),                    1, 'Correct pass/fail' );
is( $contam->comments(), 'this is a test', 'Correct comment text' );
isa_ok( $contam->genome_factor(), 'HASH', 'Genome factors returned in hash' );
isa_ok( $contam->contaminant_count(), 'HASH',
        'Contamination counts returned in hash' );
is( $contam->genome_factor_raw(),
    '{"Homo_sapiens": 12.46, "Danio_rerio": 4.7}',
    'Correct raw genome factors' );
is( $contam->contaminant_count_raw(),
    '{"Homo_sapiens": 345211, "Danio_rerio": 5000}',
    'Correct raw contaminant counts' );

my $contam2;
lives_ok {
    $contam2 =
        npg_qc::autoqc::db::checks::contamination->new( id_run   => 3300,
                                                        position =>    7,
                                                        tag_index => -1,
                                              )
}
'Successfully query database';

is( $contam2->id_contamination(), 1, 'Correct table index' );
is ( $contam2->tag_index(), undef, 'Tag index undefined' );

my $contam3 = npg_qc::autoqc::db::checks::contamination->save_from_json(
               q{ { "id_run": 3301,
                    "position": 8,
                    "path": "/tmp",
                    "__CLASS__":"npg_qc::autoqc::results::contamination-5467",
                    "aligner_version": "Bowtie 1.2.3",
                    "reference_version": "Composite_2010_13_33",
                    "pass": 1,
                    "comments": "This, too, is a test",
                    "genome_factor": {"phiX": 1.00, "Homo_sapiens": 0.23},
                    "contaminant_count": {"phiX": 10, "Homo_sapiens": 42345},
                    "read_count": 123456 }
                }
);

is( $contam3->id_contamination(), 2, 'Successfully saved from JSON string' );

1;
