#########
# Author:        jo3
# Maintainer:    $Author: ajb $
# Created:       2008-09-02
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 60-autoqc-db-checks-ref_match.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/60-autoqc-db-checks-ref_match.t $
#

use strict;
use warnings;
use Test::More tests => 18;
use Test::Exception;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw( trackingnpg core );
use t::api::util;
use Carp;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/msx; $r; };

local $ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::checks::ref_match');


my $util = t::api::util->new(
            {
                fixtures  => 1,
                dbh       => undef,
                useragent => undef,
            }
);

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db = {
    dsn      => "DBI:mysql:database=$dbname;host=$dbhost;port=$dbport",
    username => $user,
    password => $password,
};

npg_qc::autoqc::db::Schema->load_schema($db);

my $ref_match =
    npg_qc::autoqc::db::checks::ref_match->new( id_ref_match => 1 );

my @attributes = $ref_match->meta->get_all_attributes();

# Uncomment for debugging
#foreach my $attr (@attributes) {
#    my $name = $attr->name();
#    print $name, "\t" || carp;
#    print $attr->type_constraint, "\t" || carp;
#    print $ref_match->$name(), "\n" || carp;
#}

is( scalar @attributes,                  17, 'Correct attribute count' );
is( $ref_match->id_run(),              5425, 'Correct id run' );
is( $ref_match->position(),               2, 'Correct position' );
is( $ref_match->sample_read_count(),   4000, 'Number of reads sampled' );
is( $ref_match->sample_read_length(),    30, 'Length of sampled reads' );
is( $ref_match->path(),              '/tmp', 'Correct path' );
is( $ref_match->pass(),                   1, 'Correct pass/fail' );
is( $ref_match->comments(),        'a test', 'Correct comment text' );
is( $ref_match->aligner_version(), 'Bowtie 0.12.1',
    'Correct aligner version' );

isa_ok( $ref_match->reference_version(), 'HASH',
        'Reference versions returned in hash' );
isa_ok( $ref_match->aligned_read_count(), 'HASH',
        'Numbers of aligned reads returned in hash' );

is( $ref_match->reference_version_raw(),
    '{"Homo_sapiens": "1000Genomes", "Danio_rerio": "zv9"}',
    'Correct reference versions' );
is( $ref_match->aligned_read_count_raw(),
    '{"Homo_sapiens": 341, "Danio_rerio": 500}',
    'Correct aligned read counts' );

my $ref_match2;
lives_ok {
    $ref_match2 = npg_qc::autoqc::db::checks::ref_match->new(
                    id_run    => 5425,
                    position  =>    2,
                    tag_index =>   -1,
    );
}
'Successfully query database with run details';

is( $ref_match2->id_ref_match(), 1,     'Correct table index' );
is( $ref_match2->tag_index(),    undef, 'Tag index undefined' );


my $rm_json = <<'EOJSON';
{ "id_run": 3301,
   "position": 8,
   "path": "/tmp",
   "__CLASS__":"npg_qc::autoqc::results::ref_match-5467",
   "aligner_version": "Bowtie 1.2.3",
   "reference_version": {"phiX": "Made_Up", "Man": "Test"},
   "pass": 1,
   "comments": "This, too, is a test",
   "aligned_read_count": {"phiX": 10, "Man": 45},
   "sample_read_count": 12356,
   "sample_read_length": 43 }
EOJSON


my $ref_match3 =
    npg_qc::autoqc::db::checks::ref_match->save_from_json($rm_json);

is( $ref_match3->id_ref_match(), 2, 'Successfully saved from JSON string' );

1;
