#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-09-24
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 60-autoqc-db-checks-split_stats.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/60-autoqc-db-checks-split_stats.t $
#

use strict;
use warnings;
use Test::More tests => 17;
use Test::Exception;
use English qw(-no_match_vars);
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use t::useragent;
use Carp;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mx; $r; };

$ENV{dev} = 'test';

use_ok('npg_qc::autoqc::db::checks::split_stats');

my $util = t::api::util->new({fixtures => 1, dbh => undef, useragent => undef});

my $dbname   = $util->dbname();
my $dbhost   = $util->dbhost();
my $dbport   = $util->dbport();
my $user     = $util->dbuser();
my $password = $util->dbpass();
my $db ={ dsn=>"DBI:mysql:database=$dbname;host=$dbhost;port=$dbport", username=>$user, password => $password };

npg_qc::autoqc::db::Schema->load_schema($db);

{
  my $split_stats = npg_qc::autoqc::db::checks::split_stats->new(id_split_stats => 1);

  my @attributes = $split_stats->meta->get_all_attributes();

#  foreach my $attr (@attributes){
#    my $name = $attr->name();
#    print $name, "\t";
#    print $attr->type_constraint, "\t";
#    print $split_stats->$name(), "\n";
#  }

  is(scalar @attributes, 26, 'correct number of attributes');
  is($split_stats->id_run, 3500, 'correct id run');
  is($split_stats->position, 1, 'correct position');

}

{
  my $split_stats;
  lives_ok {
    $split_stats = npg_qc::autoqc::db::checks::split_stats->new(id_run => 3500, position =>1, tag_index => -1, ref_name =>'human')
  } 'no croak when query database using id_run 3500 position 1';
  is($split_stats->id_split_stats(), 1, 'correct primary key for run 3500 position 1 and human splitting');
  is ($split_stats->tag_index, undef, 'tag index undefined');
  
  my $coverages = $split_stats->split_stats_coverages();
  isa_ok( $coverages, 'Fey::Object::Iterator::FromSelect');
  is($coverages->next()->coverage, 0.1234, 'correct first coverage');
  is(scalar keys %{$split_stats->alignment_coverage1()}, 2, 'correct number of chromosomes');
  is($split_stats->alignment_coverage2()->{'y'}, 0.1237, 'correct coverage value for chromosome y in second alignment');
}

{
 my $split_stats = npg_qc::autoqc::db::checks::split_stats->save_from_jsonfile('t/data/autoqc/3807_1_1-3807_1_2_human_split_stats.json');
 is($split_stats->id_split_stats(), 2, 'data saved from the json file and the primary key is 2');

 $split_stats->split_stats_coverages();
 is($split_stats->split_stats_coverages()->next()->coverage, 0.0006, 'correct first coverage value');
}

{
 my $split_stats = npg_qc::autoqc::db::checks::split_stats->save_from_jsonfile('t/data/autoqc/3807_2_1-3807_2_2_human_split_stats.json');
 is($split_stats->id_split_stats(), 3, 'data saved from the json file and the primary key is 3');
 is($split_stats->id_run, 3807, 'correct id_run');
 is($split_stats->position, 2, 'correct position');
 $split_stats->split_stats_coverages();
 is($split_stats->split_stats_coverages()->next()->coverage, 0.0006, 'correct first coverage value');
}

1;
__END__
