#########
# Author:        dj3
# Maintainer:    $Author: gq1 $
# Created:       3 December 2009
# Last Modified: $Date: 2009-11-11 15:03:38 +0000 (Wed, 11 Nov 2009) $
# Id:            $Id: 60-autoqc-results-qX_yield.t 7140 2009-11-11 15:03:38Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/branches/prerelease-19.0/t/60-autoqc-results-qX_yield.t $
#

use strict;
use warnings;
use Test::More tests => 56;
use Test::Deep;
use Test::Exception;
use English qw(-no_match_vars);
use Carp;

$ENV{dev} ||= q{dev};

our $VERSION = do { my ($r) = q$Revision: 7140 $ =~ /(\d+)/mx; $r; };

use_ok ('npg_qc::autoqc::results::sequence_error');

{
    my $r = npg_qc::autoqc::results::sequence_error->new(id_run => 2, path => q[mypath], position => 1);
    isa_ok ($r, 'npg_qc::autoqc::results::sequence_error');
}

{
    my $r = npg_qc::autoqc::results::sequence_error->new(id_run => 2, path => q[mypath], position => 1);
    is ($r->check_name, q[sequence mismatch], 'check name changed to sequence mismatch');
    $r = npg_qc::autoqc::results::sequence_error->new(id_run => 2, path => q[mypath], position => 1, sequence_type => 'spiked_phix');
    ok ($r->can(q[sequence_type]), 'object can call sequence_type method');
    is ($r->sequence_type, q[spiked_phix], 'spiked phix sequence type');
    is ($r->check_name, q[sequence mismatch spiked phix], 'check name for spiked phix type');
}


{
    my $r;
    lives_ok {
      $r = npg_qc::autoqc::results::sequence_error->load('t/data/autoqc/4078_1.sequence_error.json');
    } q(load serialised empty result);
    isa_ok ($r, 'npg_qc::autoqc::results::sequence_error');
    my $url;
    lives_ok { $url = $r->forward_google_chart; } q(graph url creation);
    ok( !$url, q(url is empty));

    my @urls;
    lives_ok { @urls = $r->google_charts; } q(generate two urls at once);

    my $ape;
    lives_ok { $ape = $r->reverse_average_percent_error; } q(reverse_average_percent_error run);
    cmp_ok($ape, q(eq), q(nan), q(reverse_average_percent_error value));
}
{
    my $r;
    lives_ok {
      $r = npg_qc::autoqc::results::sequence_error->load('t/data/autoqc/4068_3.sequence_error.json');
    } q(load serialised valid result);
    isa_ok ($r, 'npg_qc::autoqc::results::sequence_error');
    my $url;
    lives_ok { $url = $r->forward_google_chart; } q(graph url creation);

  my $expected_url = q{http://chart.apis.google.com/chart?chbh=4,1,1&chco=9EAAFF&chd=t:0.57,0.30,0.38,0.23,0.25,0.34,0.30,0.34,0.25,0.30,0.27,0.28,0.23,0.38,0.27,0.30,3.94,0.20,0.30,0.26,0.26,0.61,0.28,0.25,0.25,0.25,0.26,0.24,0.31,0.24,0.20,0.33,0.27,0.34,0.34,0.36,0.38,0.40,0.28,0.55,0.33,0.38,0.47,0.37,0.51,0.34,0.38,0.30,0.33,0.44,0.38,0.49,0.58,0.36,0.37,0.34,0.53,0.36,0.49,0.47,0.66,0.62,0.49,0.52,0.56,0.62,0.51,0.56,0.59,0.53,0.72,0.69,0.72,0.70,0.55,0.79,0.97,3.11,0.98,0.88,1.08,0.91,1.07,0.75,0.97,0.95,1.14,1.13,1.29,1.17,1.61,1.38,1.63,1.40,1.58,1.89,1.77,1.95,1.65,1.98,2.14,2.52,2.12,2.50,3.01,2.74,2.85,3.95&chds=0,3.95&chs=585x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+4068_3_1.fastq&chxr=0,0,108,13|1,0,3.95,0&chxt=x,y};
    is( $url, $expected_url, q(url is as expected));
    my $ape;
    lives_ok { $ape = $r->reverse_average_percent_error; } q(reverse_average_percent_error run);
    cmp_ok($ape, q(==), 1.58, q(reverse_average_percent_error value));

    my @urls;
    lives_ok { @urls = $r->google_charts; } q(generate two urls at once);
    is (scalar @urls, 2 , '2 image urls generated');
    $expected_url =    'http://chart.apis.google.com/chart?chbh=4,1,1&chco=9EAAFF&chd=t:0.57,0.30,0.38,0.23,0.25,0.34,0.30,0.34,0.25,0.30,0.27,0.28,0.23,0.38,0.27,0.30,3.94,0.20,0.30,0.26,0.26,0.61,0.28,0.25,0.25,0.25,0.26,0.24,0.31,0.24,0.20,0.33,0.27,0.34,0.34,0.36,0.38,0.40,0.28,0.55,0.33,0.38,0.47,0.37,0.51,0.34,0.38,0.30,0.33,0.44,0.38,0.49,0.58,0.36,0.37,0.34,0.53,0.36,0.49,0.47,0.66,0.62,0.49,0.52,0.56,0.62,0.51,0.56,0.59,0.53,0.72,0.69,0.72,0.70,0.55,0.79,0.97,3.11,0.98,0.88,1.08,0.91,1.07,0.75,0.97,0.95,1.14,1.13,1.29,1.17,1.61,1.38,1.63,1.40,1.58,1.89,1.77,1.95,1.65,1.98,2.14,2.52,2.12,2.50,3.01,2.74,2.85,3.95&chds=0,8.69&chs=585x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+4068_3_1.fastq&chxr=0,0,108,13|1,0,8.69,0&chxt=x,y';
    is( $urls[0], $expected_url, q{first of the 2 urls is correct} );
    $expected_url =  'http://chart.apis.google.com/chart?chbh=4,1,1&chco=9EAAFF&chd=t:0.49,0.39,0.37,0.25,0.27,0.27,0.27,0.35,0.29,0.49,0.25,0.22,0.25,0.34,0.30,0.76,0.44,0.30,0.51,0.59,0.29,0.34,0.64,0.46,0.76,0.22,0.42,0.76,0.24,0.27,0.24,0.44,0.35,0.37,0.29,0.39,0.40,0.32,0.22,0.32,0.34,0.39,0.46,0.46,0.30,0.34,0.44,0.54,0.35,0.32,0.44,0.42,0.56,0.49,0.47,0.52,0.49,0.47,0.74,0.52,0.54,0.74,0.67,0.69,0.84,0.86,0.74,0.88,1.79,1.23,1.16,0.91,1.06,1.01,1.33,1.25,1.37,1.87,1.72,1.69,1.82,1.57,1.97,1.97,2.07,2.31,2.56,2.75,2.73,3.05,3.79,3.56,3.46,3.93,3.51,4.06,4.76,4.99,5.75,5.48,6.11,5.97,6.48,6.95,7.35,8.69,8.38,8.53&chds=0,8.69&chs=585x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+4068_3_2.fastq&chxr=0,0,108,13|1,0,8.69,0&chxt=x,y';
    is( $urls[1], $expected_url, q{second of the 2 urls is correct} );
    is( $r->chart_legend(), undef, q{no url returned as only 1 set of data} );
}
{
    my $r;
    lives_ok {
      $r = npg_qc::autoqc::results::sequence_error->load('t/data/autoqc/9999_1.sequence_error.json');
    } q(load serialised valid result);
    isa_ok ($r, 'npg_qc::autoqc::results::sequence_error');
    my $url;
    lives_ok { $url = $r->forward_google_chart; } q(graph url creation);

  my $expected_url = q{http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.21,0.12,0.38,0.23,0.25,0.34,0.30,0.34,0.21,0.23,0.27,0.28,0.23,0.38,0.27,0.24,0.38,0.20,0.30,0.26,0.26,0.61,0.28,0.25,0.25,0.25,0.26,0.24,0.31,0.24,0.20,0.33,0.27,0.34,0.34,0.36,0.38,0.40,0.28,0.55,0.33,0.38,0.47,0.37,0.51,0.34,0.38,0.30,0.33,0.44,0.38,0.49,0.58,0.36,0.37,0.34,0.53,0.36,0.49,0.47,0.66,0.38,0.25,0.28,0.32,0.38,0.27,0.32,0.36,0.30,0.49,0.45,0.49,0.46,0.31,0.56,0.74,0.44,0.75,0.64,0.84,0.68,0.83,0.51,0.74,0.71,0.90,0.89,1.06,0.94,1.38,1.14,1.39,1.16,1.34,1.65,1.53,1.71,1.41,1.74,1.90,2.28,1.89,2.27,2.78,2.37,2.37,2.92|0.36,0.18,0.00,0.00,0.00,0.00,0.00,0.00,0.04,0.07,0.00,0.00,0.00,0.00,0.00,0.06,3.56,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,2.67,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.37,0.47,1.03&chds=0,3.95&chs=585x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+9999_1_1.fastq&chxr=0,0,108,13|1,0,3.95,0&chxt=x,y};
    is( $url, $expected_url, q(forward_google_chart url is as expected));
    my $ape;
    lives_ok { $ape = $r->reverse_average_percent_error; } q(reverse_average_percent_error run);
    cmp_ok($ape, q(==), 1.58, q(reverse_average_percent_error value));

    my @urls;
    lives_ok { @urls = $r->google_charts; } q(generate two urls at once);
    is (scalar @urls, 2 , '2 image urls generated');

    $expected_url =    'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.21,0.12,0.38,0.23,0.25,0.34,0.30,0.34,0.21,0.23,0.27,0.28,0.23,0.38,0.27,0.24,0.38,0.20,0.30,0.26,0.26,0.61,0.28,0.25,0.25,0.25,0.26,0.24,0.31,0.24,0.20,0.33,0.27,0.34,0.34,0.36,0.38,0.40,0.28,0.55,0.33,0.38,0.47,0.37,0.51,0.34,0.38,0.30,0.33,0.44,0.38,0.49,0.58,0.36,0.37,0.34,0.53,0.36,0.49,0.47,0.66,0.38,0.25,0.28,0.32,0.38,0.27,0.32,0.36,0.30,0.49,0.45,0.49,0.46,0.31,0.56,0.74,0.44,0.75,0.64,0.84,0.68,0.83,0.51,0.74,0.71,0.90,0.89,1.06,0.94,1.38,1.14,1.39,1.16,1.34,1.65,1.53,1.71,1.41,1.74,1.90,2.28,1.89,2.27,2.78,2.37,2.37,2.92|0.36,0.18,0.00,0.00,0.00,0.00,0.00,0.00,0.04,0.07,0.00,0.00,0.00,0.00,0.00,0.06,3.56,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,2.67,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.24,0.37,0.47,1.03&chds=0,8.69&chs=585x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+9999_1_1.fastq&chxr=0,0,108,13|1,0,8.69,0&chxt=x,y';
    is( $urls[0], $expected_url, q{forward chart is correct} );
    $expected_url =    'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.35,0.35,0.00,0.25,0.27,0.27,0.27,0.30,0.29,0.49,0.19,0.13,0.22,0.00,0.30,0.76,0.44,0.30,0.51,0.59,0.29,0.34,0.64,0.46,0.76,0.22,0.42,0.76,0.24,0.27,0.24,0.44,0.35,0.37,0.29,0.39,0.35,0.32,0.22,0.32,0.34,0.39,0.46,0.46,0.30,0.34,0.44,0.54,0.35,0.32,0.44,0.42,0.56,0.49,0.47,0.52,0.49,0.47,0.74,0.52,0.54,0.74,0.67,0.69,0.84,0.86,0.74,0.88,0.94,1.23,1.16,0.91,1.06,1.01,1.33,1.25,1.20,1.70,1.55,1.52,1.65,1.40,1.80,1.80,1.91,2.14,2.40,2.58,2.56,2.88,3.63,3.39,3.29,3.76,3.34,3.90,4.59,4.82,5.58,5.31,5.94,5.80,6.31,6.78,7.19,8.52,8.21,8.37|0.13,0.03,0.37,0.00,0.00,0.00,0.00,0.05,0.00,0.00,0.07,0.08,0.03,0.34,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.05,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.84,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17,0.17&chds=0,8.69&chs=585x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+9999_1_2.fastq&chxr=0,0,108,13|1,0,8.69,0&chxt=x,y';
    is( $urls[1], $expected_url, q{reverse chart is correct} );
    lives_ok { $url = $r->reverse_google_chart; } q(graph url creation);
    is( $url, $expected_url, q{reverse_google_chart correct} );

    is( $r->chart_legend(), q{http://chart.apis.google.com/chart?chdl=A/C/G/T|N&chco=4D89F9,DB4400&cht=bvs&chs=70x250}, q{chart_legend with multiple quality bins} );

}
{
    my @result_chart_urls;
    foreach my $position ( 1..8 ) {
      lives_ok {
        push @result_chart_urls, npg_qc::autoqc::results::sequence_error->load(
          't/data/autoqc/sequence_error/6376_' . $position . '.sequence_error.json'
        )->google_charts;
      } qq{generate 2 urls at once for position $position};
    }

    my $expected_chart_urls = [   'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.46,0.51,0.32,0.37,0.38,0.30,0.30,0.30,0.22,0.26,0.30,0.40,0.30,0.31,0.32,0.24,0.33,0.40,0.32,0.37,0.33,0.34,0.30,0.35,0.35,0.40,0.41,0.38,0.46,0.51,0.28,0.64,0.64,0.65,0.78,0.72,0.82,0.68,0.62,0.79,0.63,0.66,0.76,0.64,0.66,0.66,-0.22,0.17,0.95,0.63,0.91,0.65,0.84,0.91,0.79,0.75,0.82,0.84,0.87,0.78,0.90,0.86,1.00,0.85,0.97,0.96,0.86,0.96,1.02,1.03,1.00,1.22,1.01,0.95,1.21,1.23,1.25,1.20,1.37,1.51,1.20,1.25,1.25,1.50,1.37,1.56,1.45,1.53,1.67,1.80,1.81,1.75,1.84,1.60,2.05,2.18,2.25,2.30,2.43,2.85|0.29,0.13,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.01,0.00,0.01,0.00,0.00,0.02,0.02,0.01,0.01,0.02,0.02,0.03,0.03,0.01,0.01,0.01,100.22,51.15,0.00,0.00,0.00,0.01,0.01,0.00,0.00,0.00,0.00,0.01,0.15,0.14,0.12,0.13,0.15,0.15,0.12,0.14,0.16,0.14,0.14,0.15,0.14,0.14,0.14,0.12,0.16,0.16,0.16,0.17,0.16,0.17,0.17,0.16,0.16,0.17,0.16,0.16,0.17,0.16,0.17,0.16,0.16,0.16,0.16,0.14,0.14,0.13,0.14,0.13,0.14,0.17&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_1_1.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.43,0.35,0.32,0.30,0.42,0.29,0.26,0.38,0.38,0.32,0.30,0.33,0.25,0.33,0.33,0.34,0.33,0.32,0.35,0.40,0.41,0.32,0.37,0.37,0.43,0.36,0.40,0.40,0.33,0.47,0.47,0.78,0.76,0.73,0.65,0.69,0.75,0.76,0.61,0.65,0.71,0.73,0.89,0.64,0.77,0.84,0.74,0.75,0.79,0.82,0.93,0.93,0.89,0.98,1.03,0.94,0.83,1.00,0.93,0.98,1.10,1.03,1.09,1.29,1.01,1.04,1.08,1.36,1.23,1.30,1.36,1.28,1.33,1.39,1.43,1.66,1.53,1.48,1.90,1.69,1.52,1.67,1.68,1.74,1.81,1.67,1.91,1.84,2.18,2.08,2.07,2.47,2.22,2.48,2.48,2.57,2.93,2.96,2.97,4.05|0.01,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.01,0.01,0.00,0.00,0.01,0.00,0.01,0.00,0.00,0.00,0.00,0.01,0.01,0.03,0.01,0.00,0.00,0.00,0.00,0.00,0.05,0.06,0.07,0.06,0.06,0.12,0.10,0.11,0.12,0.09,0.12,0.11,0.13,0.12,0.12,0.11,0.11,0.11,0.11,0.11,0.13,0.11,0.13,0.14,0.14,0.13,0.11,0.13,0.12,0.15,0.15,0.16,0.12,0.13,0.12,0.12,0.13,0.12,0.13,0.12,0.15,0.13,0.13,0.12,0.13,0.13,0.13,0.11&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_1_2.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.70,0.55,0.52,0.45,0.34,0.43,0.41,0.53,0.38,0.36,0.52,0.37,0.39,0.50,0.42,0.57,0.38,0.40,0.42,0.32,0.48,0.59,0.39,0.43,0.59,0.59,0.47,0.48,0.56,0.50,0.52,1.00,0.97,0.83,1.24,0.93,0.94,1.00,1.13,0.99,1.26,1.21,1.05,1.08,1.20,1.35,-0.27,0.60,1.26,1.18,1.17,1.32,1.33,1.34,1.50,1.42,1.47,1.61,1.36,1.53,1.73,1.73,1.80,1.55,1.60,1.94,1.80,2.04,2.09,1.91,2.20,2.20,2.13,2.47,2.31,2.20,2.37,2.68,2.52,2.75,2.70,2.64,2.75,2.68,2.93,2.85,3.03,3.10,3.21,3.50,3.30,3.78,3.63,3.89,4.02,4.16,4.61,4.79,4.87,5.90|0.33,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.01,0.01,0.01,0.00,0.01,0.00,0.01,0.01,0.01,0.00,0.00,0.00,0.01,0.01,100.27,51.28,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.01,0.00,0.14,0.14,0.13,0.10,0.11,0.12,0.14,0.13,0.14,0.12,0.14,0.14,0.14,0.14,0.14,0.11,0.10,0.12,0.14,0.12,0.14,0.14,0.14,0.14,0.14,0.14,0.14,0.14,0.14,0.14,0.14,0.14,0.12,0.14,0.14,0.12,0.11,0.14,0.10,0.12,0.14,0.11&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_2_1.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.91,0.61,0.60,0.52,0.49,0.51,0.46,0.54,0.51,0.46,0.48,0.52,0.48,0.41,0.52,0.53,0.48,0.41,0.55,0.49,0.53,0.55,0.40,0.53,0.53,0.58,0.52,0.86,0.73,0.74,0.70,1.14,1.09,1.23,1.21,1.17,1.21,1.31,1.24,1.41,1.64,1.24,1.24,1.48,1.37,1.53,1.43,1.31,1.47,1.63,1.47,1.65,1.81,1.82,1.73,1.86,1.76,1.91,1.95,2.13,2.16,2.33,2.15,2.34,1.98,2.19,2.32,2.34,2.71,2.54,2.70,2.41,2.83,2.64,3.02,2.84,3.14,3.13,3.02,3.68,3.48,3.25,3.46,3.62,4.04,3.55,4.10,4.15,4.46,4.21,4.35,4.67,4.91,4.94,5.19,5.23,5.53,6.28,6.19,6.70|0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_2_2.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.33,0.38,0.19,0.26,0.34,0.28,0.28,0.27,0.34,0.25,0.32,0.27,0.26,0.31,0.33,0.32,0.39,0.24,0.34,0.27,0.22,0.28,0.26,0.32,0.32,0.27,0.50,0.34,0.33,0.24,0.33,0.51,0.77,0.46,0.57,0.54,0.48,0.63,0.50,0.60,0.65,0.56,0.68,0.54,0.71,0.63,-0.11,0.21,0.77,0.73,0.69,0.47,0.89,0.68,0.69,0.73,0.74,0.84,0.92,0.98,0.96,0.88,0.79,0.88,0.94,0.95,1.02,1.04,1.05,1.16,1.25,1.16,1.04,1.16,1.29,1.00,1.28,1.24,1.24,1.37,1.25,1.38,1.64,1.77,1.40,1.59,1.57,1.66,1.66,1.71,1.95,2.18,1.95,2.13,2.10,2.15,2.05,2.51,2.37,2.95|0.25,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.02,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.02,0.00,0.00,0.00,0.00,0.01,100.11,50.24,0.00,0.00,0.00,0.01,0.01,0.01,0.01,0.00,0.00,0.01,0.09,0.09,0.09,0.09,0.09,0.09,0.09,0.09,0.09,0.09,0.09,0.08,0.09,0.08,0.09,0.09,0.08,0.08,0.10,0.10,0.09,0.10,0.09,0.11,0.09,0.09,0.09,0.09,0.09,0.09,0.09,0.10,0.09,0.09,0.09,0.09,0.08,0.09,0.09,0.08,0.08,0.08&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_3_1.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.41,0.38,0.27,0.40,0.54,0.40,0.28,0.48,0.35,0.29,0.42,0.44,0.33,0.37,0.56,0.36,0.50,0.32,0.41,0.44,0.39,0.35,0.49,0.31,0.39,0.41,0.56,0.48,0.55,0.35,0.43,0.82,0.77,0.73,0.82,0.69,0.91,0.90,0.88,0.76,0.79,0.83,0.81,0.85,0.88,0.90,0.99,1.11,0.97,1.20,0.96,0.97,0.94,1.11,0.99,1.16,1.19,1.27,1.07,1.20,1.24,1.41,1.46,1.27,1.50,1.49,1.52,1.48,1.57,1.71,1.58,1.69,1.70,1.71,1.82,1.96,2.04,2.06,1.94,1.93,2.33,2.30,2.49,2.40,2.60,2.73,2.93,2.73,2.65,3.03,3.08,3.24,3.53,3.42,3.59,4.00,4.08,4.18,3.89,4.46|0.01,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_3_2.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.57,0.35,0.22,0.35,0.29,0.31,0.34,0.31,0.24,0.36,0.41,0.30,0.36,0.31,0.25,0.27,0.31,0.29,0.31,0.32,0.26,0.22,0.30,0.41,0.35,0.33,0.33,0.32,0.37,0.37,0.24,0.42,0.60,0.47,0.42,0.50,0.65,0.60,0.68,0.63,0.51,0.67,0.47,0.66,0.76,0.68,-0.17,0.24,0.73,0.71,0.94,0.65,0.69,0.76,0.76,0.82,0.89,0.76,0.64,0.77,0.87,0.91,0.98,1.13,0.86,1.22,1.27,1.03,1.13,1.32,1.03,1.40,1.25,1.43,1.61,1.51,1.49,1.53,1.45,1.76,1.66,1.77,1.78,1.79,1.87,2.02,2.21,2.19,2.32,2.15,2.21,2.32,2.57,2.42,2.62,2.82,3.02,3.08,3.26,4.01|0.24,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.01,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,100.17,49.91,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.02,0.02,0.02,0.03,0.02,0.02,0.02,0.02,0.02,0.03,0.03,0.03,0.03,0.03,0.02,0.02,0.02,0.02,0.03,0.03,0.03,0.03,0.03,0.02,0.03,0.03,0.02,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.01,0.02,0.02,0.03,0.01&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_4_1.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.58,0.35,0.42,0.23,0.28,0.30,0.35,0.17,0.25,0.25,0.39,0.34,0.23,0.37,0.28,0.19,0.24,0.36,0.34,0.43,0.31,0.38,0.37,0.31,0.30,0.36,0.45,0.47,0.40,0.56,0.37,0.63,0.51,0.54,0.60,0.56,0.58,0.63,0.67,0.68,0.62,0.66,0.67,0.54,0.89,0.76,0.85,0.71,0.80,0.73,0.76,0.87,0.97,0.96,0.79,0.83,0.88,0.98,1.24,1.06,1.06,1.19,0.98,1.28,1.37,1.28,1.18,1.33,1.32,1.28,1.28,1.46,1.33,1.48,1.75,2.03,1.72,1.81,1.78,1.91,2.09,1.93,1.97,2.05,2.28,2.25,2.45,2.69,2.49,3.08,2.83,2.81,3.05,3.33,3.53,3.25,3.51,3.70,4.17,4.62|0.00,0.02,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_4_2.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.39,0.52,0.36,0.32,0.27,0.37,0.41,0.18,0.28,0.29,0.35,0.26,0.18,0.24,0.22,0.33,0.24,0.35,0.31,0.25,0.30,0.33,0.26,0.29,0.38,0.22,0.32,0.32,0.39,0.43,0.30,0.45,0.47,0.56,0.66,0.58,0.65,0.62,0.78,0.61,0.67,0.64,0.69,0.63,0.59,0.69,-0.16,0.62,0.55,0.57,0.59,0.73,0.69,0.73,0.64,0.62,0.51,0.84,0.72,0.92,0.72,1.12,0.84,0.89,0.89,1.01,0.84,0.85,0.97,0.97,1.12,0.92,1.11,1.15,1.06,1.15,1.12,1.01,1.18,1.31,1.39,1.29,1.47,1.41,1.32,1.57,1.41,1.58,1.44,1.65,1.67,1.61,1.68,1.61,2.14,2.13,2.31,2.22,2.44,2.78|0.29,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.03,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.08,0.06,0.00,0.07,0.05,0.06,0.06,0.03,0.00,0.02,0.01,0.04,0.00,0.00,0.02,100.16,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.09,0.09,0.08,0.08,0.08,0.09,0.09,0.08,0.09,0.09,0.09,0.09,0.08,0.09,0.09,0.08,0.08,0.08,0.11,0.09,0.10,0.09,0.09,0.09,0.09,0.09,0.10,0.09,0.09,0.09,0.09,0.09,0.09,0.09,0.09,0.08,0.09,0.09,0.09,0.09,0.09,0.08&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_5_1.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.55,0.32,0.43,0.33,0.34,0.38,0.34,0.30,0.36,0.33,0.23,0.33,0.36,0.28,0.28,0.26,0.23,0.26,0.38,0.25,0.40,0.27,0.32,0.33,0.27,0.39,0.31,0.39,0.33,0.42,0.37,0.61,0.60,0.59,0.57,0.50,0.62,0.69,0.64,0.54,0.65,0.62,0.64,0.66,0.76,0.70,0.70,0.70,0.89,0.70,0.75,0.93,0.89,0.77,0.90,0.96,0.91,0.87,0.91,0.86,0.85,1.02,1.08,0.99,1.07,0.95,1.00,1.10,1.12,1.19,1.17,1.39,1.40,1.22,1.19,1.27,1.50,1.50,1.57,1.85,1.76,1.66,1.39,1.90,1.66,1.87,2.09,1.86,2.03,2.09,2.35,2.27,2.47,2.43,2.62,2.89,2.63,2.76,2.97,3.69|0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_5_2.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.51,0.53,0.34,0.39,0.29,0.25,0.30,0.27,0.28,0.22,0.23,0.24,0.25,0.26,0.25,0.38,0.31,0.30,0.29,0.32,0.34,0.27,0.32,0.38,0.33,0.37,0.31,0.29,0.27,0.36,0.30,0.54,0.44,0.41,0.56,0.42,0.48,0.53,0.69,0.56,0.49,0.58,0.48,0.63,0.69,0.63,-0.16,0.62,0.66,0.63,0.63,0.65,0.61,0.64,0.66,0.71,0.73,0.71,0.69,0.74,0.71,0.77,0.77,0.85,1.03,0.88,0.79,0.96,0.95,1.00,0.97,1.01,1.14,1.26,1.18,0.94,1.14,1.29,1.15,1.06,1.29,1.22,1.17,1.33,1.28,1.54,1.38,1.45,1.69,1.84,1.62,1.73,1.79,1.92,2.27,1.98,2.23,2.40,2.28,2.91|0.28,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.01,0.03,0.04,0.00,0.02,0.00,0.00,0.02,0.00,0.02,0.02,0.00,0.02,0.00,0.01,100.16,0.00,0.00,0.00,0.00,0.00,0.01,0.01,0.00,0.00,0.00,0.00,0.11,0.11,0.12,0.11,0.12,0.11,0.11,0.11,0.13,0.11,0.12,0.13,0.13,0.11,0.12,0.11,0.11,0.13,0.12,0.13,0.14,0.14,0.13,0.14,0.12,0.12,0.12,0.12,0.13,0.12,0.12,0.11,0.12,0.12,0.12,0.12,0.11,0.12,0.11,0.11,0.11,0.11&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_6_1.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.69,0.33,0.28,0.41,0.32,0.29,0.30,0.30,0.33,0.33,0.27,0.36,0.24,0.41,0.38,0.28,0.29,0.32,0.36,0.32,0.30,0.25,0.27,0.35,0.33,0.39,0.36,0.33,0.34,0.32,0.40,0.65,0.67,0.66,0.71,0.71,0.54,0.65,0.74,0.57,0.66,0.68,0.72,0.67,0.50,0.69,0.54,0.76,0.78,0.75,0.87,0.68,0.67,0.77,0.77,0.82,0.85,0.89,0.81,0.90,0.95,0.94,0.91,1.00,0.82,2.40,0.97,0.96,1.16,1.12,1.16,1.22,1.28,1.22,1.12,1.38,1.37,1.18,1.47,1.34,1.65,1.60,1.88,1.87,1.88,1.69,1.85,1.84,2.01,2.17,2.32,2.44,2.35,2.42,2.39,2.62,2.61,2.93,3.31,3.36|0.06,0.03,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.03,0.03,0.03,0.02,0.03,0.03,0.02,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.05,0.03,0.03,0.03,0.03,0.03,0.04,0.05,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.03,0.04,0.03,0.03,0.03,0.03,0.03,0.03,0.04,0.03,0.04,0.03&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_6_2.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.30,0.48,0.42,0.36,0.30,0.24,0.33,0.26,0.25,0.31,0.27,0.34,0.36,0.21,0.27,0.23,0.32,0.35,0.38,0.37,0.31,0.23,0.34,0.34,0.21,0.37,0.37,0.35,0.26,0.41,0.23,0.44,0.51,0.51,0.51,0.42,0.60,0.57,0.57,0.57,0.65,0.62,0.74,0.55,0.70,0.57,-0.16,0.71,0.50,0.78,0.76,0.73,0.71,0.68,0.70,0.59,0.59,0.66,0.75,0.79,0.87,0.70,0.75,0.79,0.77,0.81,0.73,0.92,0.72,0.77,0.92,1.01,0.84,1.06,1.08,1.07,0.87,1.05,1.12,1.12,1.15,1.05,1.34,1.06,1.18,1.11,1.23,1.19,1.22,1.40,1.36,1.52,1.48,1.54,1.45,1.63,1.48,1.84,1.89,2.35|0.25,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.02,0.02,0.04,0.02,0.04,0.01,0.00,0.01,0.00,0.01,0.01,0.01,0.01,0.00,100.16,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.01,0.01,0.00,0.00,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.05,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_7_1.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.57,0.42,0.43,0.33,0.39,0.29,0.33,0.31,0.34,0.34,0.26,0.28,0.30,0.34,0.28,0.45,0.42,0.42,0.36,0.35,0.39,0.36,0.53,0.36,0.34,0.33,0.46,0.46,0.35,0.38,0.49,0.83,0.75,0.90,0.58,0.59,0.75,0.65,0.71,0.59,0.69,0.79,0.80,0.81,0.69,0.69,0.71,0.76,0.64,0.78,0.75,0.71,0.73,0.73,0.81,0.99,0.94,0.97,0.84,0.89,1.11,1.04,1.01,0.94,1.06,1.04,1.10,1.20,1.18,1.16,1.02,1.16,1.14,1.22,1.31,1.39,1.36,1.41,1.31,1.53,1.43,1.56,1.48,1.64,1.80,1.78,2.11,1.84,1.95,2.21,2.16,2.22,2.21,2.15,2.25,2.53,2.25,2.71,2.84,3.86|0.02,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.07,0.08,0.08,0.08,0.08,0.09,0.08,0.09,0.09,0.08,0.08,0.08,0.08,0.09,0.08,0.09,0.09,0.10,0.09,0.08,0.08,0.09,0.08,0.09,0.09,0.08,0.09,0.10,0.10,0.09,0.10,0.10,0.10,0.08,0.09,0.08,0.08,0.08,0.08,0.09,0.10,0.08,0.09,0.09,0.09,0.09,0.09,0.09&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_7_2.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.33,0.42,0.38,0.33,0.20,0.27,0.33,0.33,0.30,0.21,0.26,0.20,0.21,0.33,0.29,0.14,0.22,0.31,0.27,0.28,0.26,0.31,0.28,0.35,0.34,0.46,0.26,0.30,0.36,0.29,0.32,0.48,0.56,0.58,0.41,0.48,0.49,0.54,0.45,0.48,0.67,0.65,0.58,0.66,0.64,0.53,-0.13,0.72,0.62,0.81,0.56,0.66,0.66,0.73,0.66,0.88,0.64,0.72,0.82,0.82,0.71,0.59,0.75,0.92,0.82,0.83,0.85,0.90,1.06,0.85,1.08,1.02,1.20,1.05,1.15,1.12,1.23,1.07,1.20,1.32,1.41,1.27,1.34,1.44,1.40,1.30,1.35,1.52,1.42,1.53,1.80,1.63,1.68,1.81,1.64,1.95,1.89,2.06,2.11,2.79|0.19,0.02,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.02,0.00,0.01,0.02,0.01,0.01,0.02,0.01,0.01,0.01,0.02,0.01,0.04,0.03,0.02,0.00,100.13,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.05,0.07,0.08,0.07,0.08,0.08,0.07,0.08,0.07,0.08,0.07,0.08,0.08,0.08,0.07,0.07,0.07,0.07,0.08,0.08,0.08,0.08,0.08,0.08,0.08,0.08,0.08,0.08,0.07,0.08,0.07,0.08,0.08,0.08,0.08,0.08,0.08,0.08,0.08,0.08,0.07,0.07,0.09&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_8_1.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y',
'http://chart.apis.google.com/chart?chbh=4,1,1&chco=4D89F9,DB4400&chd=t:0.65,0.45,0.35,0.38,0.24,0.30,0.30,0.31,0.31,0.32,0.21,0.30,0.26,0.33,0.28,0.22,0.38,0.35,0.31,0.32,0.31,0.36,0.37,0.29,0.49,0.31,0.29,0.28,0.34,0.41,0.43,0.62,0.62,0.70,0.54,0.70,0.70,0.55,0.61,0.66,0.83,0.64,0.61,0.91,0.84,0.86,0.75,0.81,0.81,0.89,1.02,0.72,0.92,0.92,0.91,0.92,0.98,0.89,1.02,1.00,1.00,0.94,0.93,1.13,1.07,1.06,1.14,1.15,1.33,1.31,1.29,1.23,1.51,1.45,1.58,1.35,1.49,1.69,1.50,1.61,1.36,1.60,1.90,2.00,2.07,2.09,2.12,2.06,2.24,2.38,2.47,2.50,2.50,2.74,2.71,2.99,3.18,3.11,3.05,3.84|0.03,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.02,0.02,0.02,0.02,0.02,0.14,0.14,0.14,0.13,0.14,0.14,0.14,0.10,0.14,0.14,0.14,0.18,0.14,0.14,0.13,0.15,0.15,0.16,0.19,0.15,0.13,0.09,0.14,0.14,0.15,0.16,0.14,0.17,0.11,0.16,0.15,0.13,0.14,0.14,0.14,0.14,0.14,0.16,0.15,0.14,0.14,0.13,0.14&chds=0,100.00&chs=545x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6376_8_2.fastq&chxr=0,0,100,12|1,0,100.00,0&chxt=x,y'];

    is_deeply( \@result_chart_urls, $expected_chart_urls, q{All 8 chart urls obtained correctly} );

}

{
    my $r;
    lives_ok {
      $r = npg_qc::autoqc::results::sequence_error->load('t/data/autoqc/9998_2.sequence_error.json');
    } q(load serialised valid result);
    isa_ok ($r, 'npg_qc::autoqc::results::sequence_error');

    is_deeply($r->forward_quality_bins(), [[0,0,0,8,4],[0,4,0,0,9],[18,7,32,11,8]], q{forward_quality_bins retrieved ok} );

    my $expected_chart_url = q{http://chart.apis.google.com/chart?chbh=4,1,1&chco=0812F7,31F613,ECF21C,DB4400&chd=t:0.00,0.00,0.00,0.09,0.05|0.00,0.05,0.00,0.00,0.11|0.21,0.08,0.38,0.13,0.09|0.36,0.18,0.00,0.00,0.00&chds=0,0.57&chs=70x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+9999_1_1.fastq&chxr=0,0,5,0|1,0,0.57,0&chxt=x,y};
    is( $r->forward_google_chart(), $expected_chart_url, q{forward google chart correct with multiple levels of data} );
    $expected_chart_url = q{http://chart.apis.google.com/chart?chbh=4,1,1&chco=0812F7,31F613,ECF21C,DB4400&chd=t:0.07,0.02,0.00,0.15,0.15|0.15,0.05,0.00,0.08,0.08|0.17,0.29,0.00,0.02,0.03|0.13,0.03,0.37,0.00,0.00&chds=0,0.52&chs=70x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+9999_1_2.fastq&chxr=0,0,5,0|1,0,0.52,0&chxt=x,y};
    is( $r->reverse_google_chart(), $expected_chart_url, q{reverse google chart correct with multiple levels of data} );

    my $expected_chart_urls = [
      q{http://chart.apis.google.com/chart?chbh=4,1,1&chco=0812F7,31F613,ECF21C,DB4400&chd=t:0.00,0.00,0.00,0.09,0.05|0.00,0.05,0.00,0.00,0.11|0.21,0.08,0.38,0.13,0.09|0.36,0.18,0.00,0.00,0.00&chds=0,0.57&chs=70x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+9999_1_1.fastq&chxr=0,0,5,0|1,0,0.57,0&chxt=x,y},
            q{http://chart.apis.google.com/chart?chbh=4,1,1&chco=0812F7,31F613,ECF21C,DB4400&chd=t:0.07,0.02,0.00,0.15,0.15|0.15,0.05,0.00,0.08,0.08|0.17,0.29,0.00,0.02,0.03|0.13,0.03,0.37,0.00,0.00&chds=0,0.57&chs=70x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+9999_1_2.fastq&chxr=0,0,5,0|1,0,0.57,0&chxt=x,y},
    ];

    my @charts = $r->google_charts();
    is_deeply( \@charts, $expected_chart_urls, q{two charts produced and ok} );

}
{
    my $r = npg_qc::autoqc::results::sequence_error->load('t/data/autoqc/6476_3.sequence_error.json');

    my @charts = $r->google_charts();

    my $expected_chart_urls = [
       'http://chart.apis.google.com/chart?chbh=4,1,1&chco=0812F7,31F613,ECF21C,DB4400&chd=t:0.17,0.10,0.08,0.13,0.20,0.11,0.15,0.08,0.15,0.14,0.16,0.03,0.11,0.05,0.24,0.14,0.13,0.13,0.07,0.44,0.16,0.10,0.09,0.11,0.08,0.10,0.08,0.05,0.15,0.10,0.09,0.10,0.15,0.11,0.13,0.17,0.42,0.18,0.15,0.13,0.09,0.11,0.11,0.11,0.10,0.11,0.11,0.11,0.10,0.36,0.12,0.10,0.11,0.11,0.14,0.18,0.13,0.18,0.11,0.16,0.31,0.09,0.11,0.10,0.23,0.10,0.14,0.09,0.11,0.12,0.19,0.15,0.08,0.15,0.15|0.03,0.05,0.02,0.13,0.04,0.02,0.06,0.05,0.04,0.08,0.00,0.01,0.02,0.00,0.10,0.06,0.01,0.08,0.01,0.31,0.33,0.00,0.07,0.09,0.04,0.19,0.00,0.04,0.13,0.89,0.24,0.03,0.27,0.03,0.02,0.11,0.62,0.02,0.06,0.02,1.05,0.02,0.08,0.03,0.01,0.03,0.01,0.03,0.01,0.36,0.06,0.04,0.02,0.02,0.06,0.02,0.00,0.03,0.00,0.07,0.37,0.04,0.05,0.04,0.14,0.03,0.14,0.06,0.05,0.04,0.13,0.03,0.02,0.06,0.17|0.01,0.00,0.01,0.09,0.09,0.03,0.03,0.10,0.02,0.14,0.01,0.01,0.02,0.01,0.14,0.12,0.01,0.17,0.02,0.20,0.20,0.00,0.16,0.16,0.03,0.26,0.01,0.06,0.23,2.80,0.33,0.07,0.64,0.14,0.08,0.18,0.95,0.16,0.18,0.36,8.57,0.13,0.60,0.14,0.14,0.15,0.14,0.16,0.13,0.54,0.27,0.19,0.21,0.22,0.36,0.12,0.15,0.28,0.20,0.17,1.88,0.25,0.24,0.81,1.04,0.23,0.80,0.33,0.38,0.23,0.69,0.39,0.35,0.32,0.65|0.30,0.00,14.01,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.01,0.00,0.01,0.00,0.00,0.00,0.02,0.00,0.00,0.00,0.01,9.10,0.01,0.00,0.00,0.00,0.00,0.00,0.02,0.02,0.00,0.00,18.63,0.00,0.01,0.00,0.02,0.03,0.01,0.00,0.00,0.00,0.03,0.00,0.00,0.00,0.00,0.00,0.00,0.02,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.01,0.00&chds=0,28.34&chs=420x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6476_3_1.fastq&chxr=0,0,75,9|1,0,28.34,0&chxt=x,y',
       'http://chart.apis.google.com/chart?chbh=4,1,1&chco=0812F7,31F613,ECF21C,DB4400&chd=t:0.53,0.23,0.14,0.11,0.17,0.10,0.25,0.18,0.10,0.08,0.13,0.26,0.13,0.11,0.21,0.04,0.09,0.07,0.15,0.14,0.12,0.10,0.07,0.13,0.07,0.38,0.09,0.09,0.14,0.10,0.20,0.13,0.19,0.19,0.15,0.17,0.16,0.15,0.14,0.09,0.09,0.16,0.18,0.09,0.12,0.13,0.11,0.13,0.16,0.20,0.62,0.14,0.11,0.16,0.21,0.17,0.16,0.16,0.09,0.15,0.15,0.16,0.17,0.16,0.16,0.19,0.11,0.15,0.22,0.18,0.15,0.27,0.25,0.32,0.76|0.10,0.07,0.02,0.04,0.00,0.00,0.18,0.42,0.02,0.00,0.02,0.58,0.02,0.01,0.05,0.01,0.01,0.01,0.05,0.02,0.00,0.01,0.02,0.03,0.01,0.15,0.00,0.03,0.01,0.07,0.10,0.08,0.20,0.16,0.04,0.08,0.04,0.05,0.07,0.02,0.03,0.04,0.11,0.02,0.03,0.05,0.07,0.03,0.06,0.04,0.36,0.03,0.03,0.05,0.25,0.18,0.04,0.03,0.03,0.05,0.07,0.29,0.04,0.05,0.05,0.25,0.11,0.09,0.07,0.07,0.07,0.31,0.09,0.19,3.25|0.06,0.07,0.02,0.05,0.03,0.04,0.20,0.38,0.02,0.04,0.05,0.64,0.03,0.04,0.11,0.04,0.07,0.04,0.17,0.07,0.11,0.09,0.04,0.07,0.07,0.25,0.17,0.15,0.11,0.11,0.27,0.25,0.27,0.31,0.14,0.39,0.24,0.19,0.24,0.27,0.23,0.16,0.34,0.23,0.18,0.28,0.33,0.24,0.34,0.27,0.92,0.20,0.30,0.24,0.54,0.53,0.29,0.21,0.31,0.19,0.25,0.83,0.36,0.37,0.32,0.71,0.40,0.35,0.34,0.47,0.44,0.94,0.55,0.42,8.58|0.13,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.12,0.14,0.26,0.27,0.01,0.01,0.01,0.01,0.01,0.00,0.01,0.01,0.02,0.00,0.01,0.03,0.02,0.00,0.00,0.00,0.02,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.01,0.00,0.00,0.01,0.01,0.01,0.01,0.01,0.01&chds=0,28.34&chs=420x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6476_3_2.fastq&chxr=0,0,75,9|1,0,28.34,0&chxt=x,y'
    ];
    is_deeply( \@charts, $expected_chart_urls, q{urls produced correctly from json file with quality bins - large low qval} );
}

{
    my $r = npg_qc::autoqc::results::sequence_error->load('t/data/autoqc/sequence_error/6467_2.sequence_error.json');

    my @charts = $r->google_charts();
    my $expected_chart_urls = [
        'http://chart.apis.google.com/chart?chbh=4,1,1&chco=0812F7,31F613,ECF21C,DB4400&chd=t:3.08,1.23,0.68,0.81,0.65,0.34,0.38,0.34,0.31,0.38,0.37,0.27,0.31,0.29,0.18,0.36,0.29,0.37,0.40,0.35,0.39,0.39,0.23,0.44,0.24,0.30,0.21,0.40,0.54,0.62,0.69,0.73,1.08,1.09,1.22,1.47,1.57,1.78,1.73,1.79,2.05,1.71,2.05,1.99,2.23,2.14,2.54,2.74,2.56,2.46,2.93,2.91,3.08,3.27,3.20,3.36,3.59,3.69,3.71,4.15,3.81,4.08,4.28,4.06,4.65,4.71,4.97,5.14,5.48,5.83,5.71,6.19,6.41,6.58,6.41|0.25,0.12,0.08,0.16,0.60,0.06,0.05,0.01,0.01,0.11,0.00,0.09,0.04,0.01,0.00,0.09,0.01,0.01,0.01,0.01,0.41,0.04,0.00,0.13,0.01,0.03,0.05,0.09,0.10,0.04,0.08,0.09,0.46,0.09,0.03,0.11,0.08,0.08,0.07,0.27,0.13,0.10,0.15,0.17,0.21,0.12,0.15,0.23,0.20,0.38,0.23,0.24,0.32,0.31,0.38,0.25,0.36,0.42,0.43,0.47,0.49,0.58,0.53,0.69,0.70,0.72,0.84,0.86,0.87,0.90,1.26,1.30,1.31,1.37,2.07|0.08,0.04,0.00,0.09,0.45,0.05,0.03,0.00,0.00,0.11,0.01,0.12,0.01,0.00,0.01,0.06,0.03,0.00,0.00,0.00,0.79,0.03,0.05,0.23,0.02,0.08,0.11,0.24,0.19,0.12,0.10,0.08,0.65,0.13,0.14,0.09,0.12,0.08,0.15,0.49,0.15,0.19,0.16,0.29,0.23,0.21,0.23,0.20,0.20,0.19,0.48,0.33,0.23,0.30,0.33,0.32,0.30,0.34,0.36,0.50,0.45,0.45,0.56,0.46,0.59,0.51,0.73,0.59,0.61,0.72,0.71,0.73,0.89,0.91,0.94|0.04,0.01,0.00,0.00,0.02,0.00,0.02,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.02,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00&chds=0,9.78&chs=420x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6467_2_1.fastq&chxr=0,0,75,9|1,0,9.78,0&chxt=x,y',
       'http://chart.apis.google.com/chart?chbh=4,1,1&chco=0812F7,31F613,ECF21C,DB4400&chd=t:5.39,1.22,0.87,0.50,0.51,0.41,0.36,0.30,0.34,0.23,0.32,0.26,0.34,0.23,0.26,0.28,0.31,0.31,0.36,0.30,0.35,0.34,0.21,0.26,0.40,0.30,0.29,0.40,0.62,0.52,0.69,0.74,1.03,1.02,1.23,1.18,1.20,1.19,1.65,1.68,1.63,1.79,1.83,1.94,2.20,2.37,2.60,2.57,2.79,2.76,2.62,2.78,3.10,3.50,3.49,3.64,3.75,3.75,4.02,4.18,4.26,4.24,4.55,4.58,4.70,4.94,5.10,5.23,5.77,6.45,6.24,6.33,6.37,7.26,7.16|0.50,0.13,0.04,0.06,0.01,0.01,0.00,0.02,0.00,0.02,0.02,0.02,0.04,0.03,0.00,0.00,0.00,0.05,0.04,0.03,0.01,0.02,0.01,0.03,0.03,0.09,0.05,0.00,0.02,0.08,0.03,0.07,0.05,0.10,0.12,0.20,0.16,0.13,0.12,0.23,0.17,0.15,0.22,0.25,0.25,0.13,0.23,0.19,0.25,0.32,0.60,0.39,0.27,0.36,0.35,0.31,0.35,0.39,0.39,0.34,0.48,0.40,0.50,0.54,0.66,0.67,0.75,0.71,0.82,0.87,0.88,1.00,0.97,1.12,1.58|0.03,0.02,0.01,0.03,0.02,0.03,0.05,0.01,0.01,0.03,0.06,0.02,0.05,0.08,0.03,0.08,0.06,0.02,0.13,0.09,0.09,0.09,0.09,0.04,0.05,0.06,0.11,0.08,0.12,0.15,0.13,0.18,0.23,0.07,0.22,0.18,0.16,0.17,0.23,0.21,0.33,0.34,0.29,0.36,0.29,0.19,0.28,0.38,0.36,0.36,0.27,0.27,0.32,0.43,0.42,0.37,0.42,0.32,0.50,0.49,0.45,0.50,0.61,0.51,0.51,0.63,0.49,0.58,0.61,0.53,0.66,0.69,0.74,0.67,1.04|0.19,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00&chds=0,9.78&chs=420x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6467_2_2.fastq&chxr=0,0,75,9|1,0,9.78,0&chxt=x,y'
    ];
    is_deeply( \@charts, $expected_chart_urls, q{urls produced correctly from json file with quality bins - large high qval} );

    my $dev_capture = $ENV{dev};

    $ENV{dev} = q{LIVE};

    @charts = $r->google_charts();
    $expected_chart_urls = [
       'http://chart.apis.google.com/chart?chbh=4,1,1&chco=0812F7,31F613,ECF21C,DB4400&chd=e:UJIDEcFTEQCOCfCOCBCfCaBxCBB5BLCWB5CaCnCSCjCjBgC4BkB9BXCnDiEDEgExHEHIH-JnKRLpLULtNaLLNaNBOlOAQnR7QvQGTKTCUJVZU7V-XfYJYRbJY7ascAajebe0gghoj2mJlWofp7rDp7,BoAyAhBCD7AZAUAEAEAuAAAlAQAEAAAlAEAEAEAECrAQAAA2AEAMAUAlApAQAhAlDAAlAMAuAhAhAdBxA2ApA-BHBXAyA-BgBTCfBgBkCFCBCfBoCWCvC0DEDNDyDdEgElEtFfFoFsF4IPIgIkI9Ni,AhAQAAAlC8AUAMAAAAAuAEAyAEAAAEAZAMAAAAAAFKAMAUBgAIAhAuBkBPAyApAhEQA2A6AlAyAhA-DNA-BPBCB5BgBXBgBTBTBPDICKBgB9CKCFB9COCWDRC8C8DqDAD3DVExD3D.EtEpExF0F9GJ,AQAEAAAAAIAAAIAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAIAAAAAAAAAEAAAAAAAAAAAAAAAAAAAAAAAAAAAAAEAAAAAAAAAAAAAAAAAAAEAAAAAAAEAAAAAAAAAAAAAAAAAAAAAAAA&chds=0,9.78&chs=420x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6467_2_1.fastq&chxr=0,0,75,9|1,0,9.78,0&chxt=x,y',
       'http://chart.apis.google.com/chart?chbh=4,1,1&chco=0812F7,31F613,ECF21C,DB4400&chd=e:jQH-FsDRDVCrCWB9COBgCFBsCOBgBsB1CBCBCWB9CSCOBXBsCnB9B5CnEDDZEgE1GvGrIDHuH2HyKyK.KqLtL-MsOZPgRAQ0SQSDRJSMUSW5W1X0YiYiaTbWb3bvdxd9evgUhXiNlvqMo0paprvfu1,DRA2AQAZAEAEAAAIAAAIAIAIAQAMAAAAAAAUAQAMAEAIAEAMAMAlAUAAAIAhAMAdAUApAyBTBCA2AyBgBHA-BcBoBoA2BgBPBoCFD7CjBxCWCSCBCSCjCjCODICnDRDiEUEYE6EpFXFsFwGiGWHUKV,AMAIAEAMAIAMAUAEAEAMAZAIAUAhAMAhAZAIA2AlAlAlAlAQAUAZAuAhAyA-A2BLBgAdBcBLBCBHBgBXCKCOB5CWB5BPB1CfCWCWBxBxCFC0CvCaCvCFDRDNC8DRD.DVDVEHDNDyD.DdEUEgE1EYGz,BPAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA&chds=0,9.78&chs=420x300&cht=bvs&chtt=Mismatch+percent+by+cycle,+6467_2_2.fastq&chxr=0,0,75,9|1,0,9.78,0&chxt=x,y'
    ];
    is_deeply( \@charts, $expected_chart_urls, q{urls produced correctly from json file with quality bins - large high qval - encoded} );

    is( $r->chart_legend(), q{http://chart.apis.google.com/chart?chdl=>=31|=<30|=<15|N&chco=0812F7,31F613,ECF21C,DB4400&cht=bvs&chs=70x250}, q{chart_legend with multiple quality bins} );

    $ENV{dev} = $dev_capture;
}
__END__

