#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-09-21
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 60-autoqc-results-tag_decode_stats.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/60-autoqc-results-tag_decode_stats.t $
#

use strict;
use warnings;
use Test::More tests => 7;
use English qw(-no_match_vars);
use Carp;
use File::Temp qw/ tempdir /;
use Perl6::Slurp;
use JSON;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mx; $r; };

use_ok ('npg_qc::autoqc::results::tag_decode_stats');

my $tempdir = tempdir( CLEANUP => 0);
{
    my $r = npg_qc::autoqc::results::tag_decode_stats->new({
                                       position => 8,
                                       id_run   => 4360,
                                 });
    isa_ok ($r, 'npg_qc::autoqc::results::tag_decode_stats');

    my $stats_output_string = slurp 't/data/autoqc/tag_decode_output.txt';
    $r->parsing_output_string($stats_output_string);    
    
    eval{
      $r->freeze();
      $r->store(qq{$tempdir/split_stats.json});
      1;
    };
    is($EVAL_ERROR, q{}, 'no croak when save data into json'); 
    
}

## tests when the decoded tags might not be found
{
    my $r = npg_qc::autoqc::results::tag_decode_stats->new({
                                       position => 8,
                                       id_run   => 4360,
                                 });
    isa_ok ($r, 'npg_qc::autoqc::results::tag_decode_stats');

    my $stats_output_string = slurp 't/data/autoqc/tag_decode_output_no_found_tags.txt';
    $r->parsing_output_string($stats_output_string);    
    
    eval{
      $r->freeze();
      $r->store(qq{$tempdir/split_stats_no_tags_found.json});
      1;
    };
    is($EVAL_ERROR, q{}, 'no croak when save data into json');

    my $hash = from_json( slurp qq{$tempdir/split_stats_no_tags_found.json} );

    my $expected_hash_structure = {
      'distribution_all' => {},
      'distribution_good' => {},
      'errors_all' => {
        '0' => '304354',
        '1' => '6627',
        'no_match' => '7512'
      },
      'errors_good' => {
        '0' => '275562',
        '1' => '1505',
        'no_match' => '2078'
      },
      'id_run' => 4360,
      'info' => {},
      'pass' => 0,
      'position' => 8,
      'tag_code' => {}
    };

    ok( exists $hash->{__CLASS__}, q{__CLASS__ key found} );
    delete $hash->{__CLASS__};

    is_deeply( $hash, $expected_hash_structure, q{expected results obtained} );

}
