#########
# Author:        mg8
# Maintainer:    $Author: ajb $
# Created:       30 July 2009
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: 70-autoqc-checks-check.t 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/70-autoqc-checks-check.t $
#


use strict;
use warnings;
use Test::More tests => 12;
use Test::Exception;
use Test::Deep;
use File::Temp qw/tempdir/;

use npg_qc::autoqc::checks::check;
use npg_qc::autoqc::checks::sequence_error;
use npg_qc::autoqc::results::result;

my $dir = tempdir( CLEANUP => 1);
my $repos = q[t/data/autoqc];


{
    my $check = npg_qc::autoqc::checks::check->new(
                                                      position  => 3,
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      id_run    =>  2549,
                                                 );
    my $r =  npg_qc::autoqc::results::result->new (
                              position  => 3,
                              path      => 't/data/autoqc/090721_IL29_2549/data',
                              id_run    => 2549,
                              tag_index => undef,
                                                  );
    $r->set_info('Check', 'npg_qc::autoqc::checks::check');
    $r->set_info('Check_version', npg_qc::autoqc::checks::check->VERSION);
                                  
    cmp_deeply($check->result, $r, 'result object created, default tag index');
}


{
    my $check = npg_qc::autoqc::checks::check->new(
                                                      position  => 3,
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      id_run    =>  2549,
                                                      tag_index => 5,
                                                 );
    my $r =  npg_qc::autoqc::results::result->new (
                              position  => 3,
                              path      => 't/data/autoqc/090721_IL29_2549/data',
                              id_run    => 2549,
                              tag_index => 5,
                                                  );
    $r->set_info('Check', 'npg_qc::autoqc::checks::check');
    $r->set_info('Check_version', npg_qc::autoqc::checks::check->VERSION);
                                  
    cmp_deeply($check->result, $r, 'result object created, tag index set');
}

{
  my $check = npg_qc::autoqc::checks::sequence_error->new({
                                                      position  => 1,
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      id_run    => 2549,
                                                      repository => $repos,
                                                     });
  is ($check->sequence_type, undef, 'sequence type unset');
  is ($check->result->sequence_type, undef, 'result sequence type unset');
}

{
  my $check = npg_qc::autoqc::checks::sequence_error->new({
                                                      position  => 1,
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      id_run    => 2549,
                                                      sequence_type => 'phix',
                                                      repository => $repos,
                                                     });
  is ($check->sequence_type, q[phix], 'sequence type is phix');
  is ($check->type_label, q[_phix], 'phix type label');
  is ($check->result->sequence_type, q[phix], 'result sequence type phix');
    
  my $forward  = q[t/data/autoqc/090721_IL29_2549/data/2549_1_1_phix.fastq];
  my $reverse  = q[t/data/autoqc/090721_IL29_2549/data/2549_1_2_phix.fastq];
  `touch $forward`;
  `touch $reverse`;
  is (join( q[ ], $check->get_input_files()), 't/data/autoqc/090721_IL29_2549/data/2549_1_1_phix.fastq t/data/autoqc/090721_IL29_2549/data/2549_1_2_phix.fastq', 'two fastqcheck input files found');
  cmp_deeply ($check->generate_filename_attr(), ['2549_1_1_phix.fastq', '2549_1_2_phix.fastq'], 'output filename structure');
  unlink $forward;
  unlink $reverse;
}

{
  my $check = npg_qc::autoqc::checks::sequence_error->new({
                                                      position  => 1,
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      id_run    => 2549,
                                                      repository => $repos,
                                                     });
  $check->write2file('dodo', $dir, q[json]);
  ok (-e join(q[/], $dir, q[2549_1.sequence_error.json]), 'result file written');
}

{
  my $check = npg_qc::autoqc::checks::sequence_error->new({
                                                      position  => 1,
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      id_run    => 2549,
                                                      sequence_type => 'phix',
                                                      repository => $repos,
                                                     });
  $check->write2file('dodo', $dir, q[json]);
  ok (-e join(q[/], $dir, q[2549_1_phix.sequence_error.json]), 'phix type result file written');
}

{
  my $check = npg_qc::autoqc::checks::sequence_error->new({
                                                      position  => 1,
                                                      path      => 't/data/autoqc/090721_IL29_2549/data',
                                                      id_run    => 2549,
                                                      tag_index => 4,
                                                      sequence_type => 'phix',
                                                      repository => $repos,
                                                     });
  $check->write2file('dodo', $dir, q[json]);
  ok (-e join(q[/], $dir, q[2549_1#4_phix.sequence_error.json]), 'phix type result file written');
}
