#########
# Author:        Marina Gourtovaia mg8@sanger.ac.uk
# Maintainer:    $Author: ajb $
# Created:       2008-06-12
# Last Modified: $Date: 2011-08-01 15:48:20 +0100 (Mon, 01 Aug 2011) $
# Id:            $Id: autoqc_util.pm 13782 2011-08-01 14:48:20Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.1/t/autoqc_util.pm $
#

package t::autoqc_util;

use strict;
use warnings;
use Carp;
use English qw(-no_match_vars);
use Data::Dumper;
use Test::More;
use XML::Simple qw(XMLin);
use Exporter;

our $VERSION = do { my ($r) = q$Revision: 13782 $ =~ /(\d+)/mx; $r; };

our @ISA       = qw(Exporter);
our @EXPORT_OK = qw(
                      write_bwa_script
                      write_samtools_script
                      insert_size_additional_modules
                   );
sub write_samtools_script {

  my ($script_path, $b1) = @_;
  
  my $fh;
  open($fh, '>', $script_path) or croak "Cannot open $script_path for writing";

  print $fh '#!/usr/local/bin/bash';
  print $fh "\n";
  print $fh '# Script to fake samtools in testing. Automatically generated by npg at ' . time;
  print $fh "\n";

  if (defined $b1) {
    print $fh "cat $b1\n";
  }else{
    print $fh "echo 'Version: 0.5.5 (r1273)'\n";
  }

  close $fh;
    
  chmod 0775, $script_path;
  return 1;
}

sub write_fastx_script {

  my ($script_path, $b1) = @_;
  
  my $fh;
  open($fh, '>', $script_path) or croak "Cannot open $script_path for writing";

  print $fh '#!/usr/local/bin/bash';
  print $fh "\n";
  print $fh '# Script to fake fastx in testing. Automatically generated by npg at ' . time;
  print $fh "\n";

  # echo the first argument $1;
  # echo number of arguments: $#;
  if (!defined $b1) {
    my $out = q[usage: fastx_reverse_complement [-h] [-r] [-z] [-v] [-i INFILE] [-o OUTFILE]
Part of FASTX Toolkit 0.0.12 by A. Gordon (gordon@cshl.edu)

   [-h]         = This helpful help screen.
   [-z]         = Compress output with GZIP.
   [-i INFILE]  = FASTA/Q input file. default is STDIN.
   [-o OUTFILE] = FASTA/Q output file. default is STDOUT.

];

    print $fh "echo '$out'";
  }

  close $fh;
    
  chmod 0775, $script_path;
  return 1;
}

sub write_bwa_script {

  my ($script_path, $s1, $bwa_version) = @_;
  
  my $fh;
  open($fh, '>', $script_path) or croak "Cannot open $script_path for writing";

  print $fh '#!/usr/local/bin/bash';
  print $fh "\n";
  print $fh '# Script to fake BWA tool in testing. Automatically generated by npg at ' . time;
  print $fh "\n";

  # echo the first argument $1;
  # echo number of arguments: $#;
  if (defined $s1) {
    print $fh 'if [ $# != 0 ]; then';
    print $fh "\n";
    print $fh 'if [ $1 == sampe ]; then';
    print $fh "\n";
    print $fh "cat $s1\n";
    print $fh "fi\n";
    print $fh "else\n";
    print $fh "echo 'Version: 0.5.5 (r1273)'\n";
    print $fh "fi\n";
  }else{
    print $fh "echo 'Version: 0.5.5 (r1273)'\n";
  }

  close $fh;
    
  chmod 0775, $script_path;
  return 1;
}

sub insert_size_additional_modules {
  my $use_fastx = shift;
  my @expected = ();
  require npg_qc::autoqc::parse::alignment;
  push @expected, join(q[ ], 'npg_qc::autoqc::parse::alignment', npg_qc::autoqc::parse::alignment->VERSION);
  require npg_common::extractor::fastq;
  push @expected, join(q[ ], q[npg_common::extractor::fastq], npg_common::extractor::fastq->VERSION);
  require npg_common::Alignment;
  push @expected, join(q[ ], q[npg_common::Alignment], npg_common::Alignment->VERSION);
  if ($use_fastx) {
    push @expected, q[FASTX Toolkit fastx_reverse_complement 0.0.12];
  }
  return @expected;
}

1;
